@@ -3,12 +3,9 @@
 #include "sysdeps.h"
 #include "tools.h"
 #include "parameters.h"
+#include "host_filesys.h"
 
-#ifdef HAVE_NEW_HEADERS
 # include <cstdlib>
-#else
-# include <stdlib.h>
-#endif
 
 int get_geometry(char *dev_path, geo_type geo) {
   return -1;
@@ -17,7 +14,7 @@ int get_geometry(char *dev_path, geo_type geo) {
 /*
  * Get the path to a user home folder.
  */
-char *getHomeFolder(char *buffer, unsigned int bufsize)
+char *HostFilesys::getHomeFolder(char *buffer, unsigned int bufsize)
 {
 	buffer[0] = '\0';	// last resort - current folder
 
@@ -33,13 +30,13 @@ char *getHomeFolder(char *buffer, unsigned int bufsize)
 /*
  * Get the path to folder with user-specific files (configuration, NVRAM)
  */
-char *getConfFolder(char *buffer, unsigned int bufsize)
+char *HostFilesys::getConfFolder(char *buffer, unsigned int bufsize)
 {
 	// local cache
 	static char path[512] = "";
 
 	if (strlen(path) == 0) {
-		char *home = getHomeFolder(path, 512);
+		char *home = HostFilesys::getHomeFolder(path, 512);
 
 		int homelen = strlen(home);
 		if (homelen > 0) {
@@ -56,9 +53,9 @@ char *getConfFolder(char *buffer, unsigned int bufsize)
 	return safe_strncpy(buffer, path, bufsize);
 }
 
-char *getDataFolder(char *buffer, unsigned int bufsize)
+char *HostFilesys::getDataFolder(char *buffer, unsigned int bufsize)
 {
-
+ 
  static char path[512]="";
  CFURLRef tosURL;
 
@@ -69,6 +66,9 @@ char *getDataFolder(char *buffer, unsigned int bufsize)
 		{
 			CFURLGetFileSystemRepresentation (tosURL, true, (uint8 *)path, 512);
 			delete (tosURL);
+			unsigned int len = strlen(path);
+			if ((len+1) < 512) 
+					strcat(path, DIRSEPARATOR);
 		}
 	}	
 	return safe_strncpy(buffer, path, bufsize);
