@@ -1,7 +1,7 @@
 /*
  * main_unix.cpp - Startup code for Unix
  *
- * Copyright (c) 2000-2004 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2000-2005 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  MJ		Milan Jurik
@@ -27,7 +27,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-
 #include "sysdeps.h"
 #include "cpu_emulation.h"
 #include "main.h"
@@ -44,15 +43,9 @@
 # include "mon.h"
 #endif
 
-#ifdef HAVE_NEW_HEADERS
 # include <cerrno>
 # include <csignal>
 # include <cstdlib>
-#else
-# include <errno.h>
-# include <signal.h>
-# include <stdlib.h>
-#endif
 
 #ifndef HAVE_STRDUP
 extern "C" char *strdup(const char *s)
@@ -89,33 +82,14 @@ void segmentationfault(int)
 	exit(0);
 }
 
+bool ThirtyThreeBitAddressing = false;
+
 static void allocate_all_memory()
 {
 #if REAL_ADDRESSING || DIRECT_ADDRESSING || FIXED_ADDRESSING
 	// Initialize VM system
 	vm_init();
 
-#if REAL_ADDRESSING
-	// Flag: RAM and ROM are contigously allocated from address 0
-	bool memory_mapped_from_zero = false;
-	
-	// Probably all OSes have problems
-	// when trying to map a too big chunk of memory starting at address 0
-	
-	// Try to allocate all memory from 0x0000, if it is not known to crash
-	if (vm_acquire_fixed(0, RAMSize + ROMSize + HWSize + FastRAMSize + RAMEnd) == true) {
-		D(bug("Could allocate RAM and ROM from 0x0000"));
-		memory_mapped_from_zero = true;
-	}
-
-	if (memory_mapped_from_zero) {
-		RAMBaseHost = (uint8 *)0;
-		ROMBaseHost = RAMBaseHost + ROMBase;
-		HWBaseHost = RAMBaseHost + HWBase;
-		FastRAMBaseHost = RAMBaseHost + FastRAMBase;
-	}
-	else
-#endif
 #if FIXED_ADDRESSING
 	if (vm_acquire_fixed((void *)FMEMORY, RAMSize + ROMSize + HWSize + FastRAMSize + RAMEnd) == false) {
 		panicbug("Not enough free memory.");
@@ -141,7 +115,35 @@ static void allocate_all_memory()
 #  endif /* HW_SISEGV */
 # endif /* EXTENDED_SIGSEGV */
 #else
+# if REAL_ADDRESSING
+	// Flag: RAM and ROM are contigously allocated from address 0
+	bool memory_mapped_from_zero = false;
+	
+	// Probably all OSes have problems
+	// when trying to map a too big chunk of memory starting at address 0
+	
+	// Try to allocate all memory from 0x0000, if it is not known to crash
+	if (vm_acquire_fixed(0, RAMSize + ROMSize + HWSize + FastRAMSize + RAMEnd) == true) {
+		D(bug("Could allocate RAM and ROM from 0x0000"));
+		memory_mapped_from_zero = true;
+	}
+
+	if (memory_mapped_from_zero) {
+		RAMBaseHost = (uint8 *)0;
+		ROMBaseHost = RAMBaseHost + ROMBase;
+		HWBaseHost = RAMBaseHost + HWBase;
+		FastRAMBaseHost = RAMBaseHost + FastRAMBase;
+	}
+	else
+# endif
 	{
+#ifdef USE_33BIT_ADDRESSING
+		// Speculatively enables 33-bit addressing
+		RAMBaseHost = (uint8*)vm_acquire(RAMSize + ROMSize + HWSize + FastRAMSize + RAMEnd, VM_MAP_DEFAULT | VM_MAP_33BIT);
+		if (RAMBaseHost != VM_MAP_FAILED)
+			ThirtyThreeBitAddressing = true;
+		else
+#endif
 		RAMBaseHost = (uint8*)vm_acquire(RAMSize + ROMSize + HWSize + FastRAMSize + RAMEnd);
 		if (RAMBaseHost == VM_MAP_FAILED) {
 			panicbug("Not enough free memory.");
@@ -282,7 +284,6 @@ int main(int argc, char **argv)
 /*
  *  Quit emulator
  */
-
 void QuitEmulator(void)
 {
 	D(bug("QuitEmulator"));
@@ -321,8 +322,6 @@ void QuitEmulator(void)
 
 	// Exit VM wrappers
 	vm_exit();
-
-	exit(0);
 }
 
 #ifdef ENABLE_MON
