@@ -1,7 +1,7 @@
 /*
  * acsifdc.cpp - Atari floppy emulation code
  *
- * Copyright (c) 2001-2004 Petr Stehlik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2001-2005 Petr Stehlik of ARAnyM dev team (see AUTHORS)
  * 
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -55,6 +55,7 @@ enum {
 /* Variables */
 
 static NCR5380 ncr5380;
+static int panic_floppy_error = 0;
 
 ACSIFDC::ACSIFDC(memptr addr, uint32 size) : BASE_IO(addr, size) {
 	drive_fd = -1;
@@ -81,7 +82,7 @@ uint16 ACSIFDC::handleReadW(memptr addr) {
 		default: value = BASE_IO::handleReadW(addr);
 	}
 
-	D(bug("Reading ACSIFDC word data from %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
+	// D(bug("Reading ACSIFDC word data from %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
 	return value;
 }
 
@@ -96,13 +97,13 @@ uint8 ACSIFDC::handleRead(memptr addr) {
 		case 5: value = getDMAData(); break;
 		case 6: value = getDMAStatus() >> 8; break;
 		case 7: value = getDMAStatus(); break;
-		case 9: value = DMAaddr >> 16; break;
+		case 9: value = DMAaddr >> 16; D(bug("reading DMA addr")); break;
 		case 0x0b: value = DMAaddr >> 8; break;
 		case 0x0d: value = DMAaddr; break;
 		case 0x0f: value = 0; break;	// missing floppy-control reg emulation
 	}
 
-	D(bug("Reading ACSIFDC data from %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
+	// D(bug("Reading ACSIFDC data from %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
 	return value;
 }
 
@@ -119,7 +120,7 @@ void ACSIFDC::handleWrite(memptr addr, uint8 value) {
 	if (addr >= getHWsize())
 		return;
 
-	D(bug("Writing ACSIFDC data to %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
+	// D(bug("Writing ACSIFDC data to %04lx = %d ($%02x) at %06x\n", addr, value, value, showPC()));
 	switch(addr) {
 		case 4: setDMASectorCount(value << 8); break;
 		case 5: setDMASectorCount(value); break;
@@ -182,10 +183,11 @@ uint16 ACSIFDC::getDMAStatus()
 
 void ACSIFDC::setDMASectorCount(uint16 vv)
 {
-	D(bug("DMA car/scr <- %x (mode=%x)", vv, dma_mode));
+	// D(bug("DMA car/scr <- %x (mode=%x)", vv, dma_mode));
 	if (dma_mode&0x10)
 	{
 		dma_scr = vv;
+		D(bug("scr = %d", dma_scr));
 	}
 	else
 	{
@@ -225,17 +227,16 @@ void ACSIFDC::setDMASectorCount(uint16 vv)
 void ACSIFDC::setDMAMode(uint16 vv)
 {
 	dma_mode = vv;
-	D(bug("DMA mode <- %04x", dma_mode));
+	// D(bug("DMA mode <- %04x", dma_mode));
 }
 
 /*************************************************************************/
 
 // parameters of default floppy (3.5" HD 1.44 MB)
 #define SECSIZE		512
-#define SPT			18
 #define SIDES		2
+#define SPT			18
 #define TRACKS		80
-#define SECTORS		(SPT * SIDES * TRACKS)
 
 void ACSIFDC::set_floppy_geometry()
 {
@@ -269,9 +270,16 @@ void ACSIFDC::set_floppy_geometry()
 		if (! valid) {
 			// bootsector contains invalid data - use our default
 			secsize = SECSIZE;
-			spt = SPT;
 			sides = SIDES;
-			sectors = SECTORS;
+
+			// for any 80 track floppy compute the sector per track
+			off_t disk_size = lseek(drive_fd, 0, SEEK_END);
+			if (disk_size % (secsize * sides * TRACKS) == 0)
+				spt = (disk_size / secsize / sides / TRACKS);
+			else // non-standard number of tracks?
+				spt = SPT; // assume HD floppy
+
+			sectors = spt * sides * TRACKS;
 		}
 
 		tracks = (sectors / spt / sides);
@@ -328,6 +336,7 @@ bool ACSIFDC::insert_floppy()
 	set_floppy_geometry();
 
 	floppy_changed = true;
+	panic_floppy_error = 0;
 	return true;
 }
 
@@ -336,19 +345,37 @@ bool ACSIFDC::is_floppy_inserted()
 	return (drive_fd >= 0);
 }
 
+bool ACSIFDC::read_file(int device, long offset, memptr address, int secsize, int count)
+{
+	if (lseek(device, offset, SEEK_SET) < 0) return false;
+	uint8 buffer[secsize];
+	for(int i=0; i<count; i++) {
+		if (::read(device, buffer, secsize) != secsize) return false;
+		memcpy(Atari2HostAddr(address), buffer, secsize);
+		address += secsize;
+	}
+	return true;
+}
+
+bool ACSIFDC::write_file(int device, long offset, memptr address, int secsize, int count)
+{
+	if (lseek(device, offset, SEEK_SET) < 0) return false;
+	uint8 buffer[secsize];
+	for(int i=0; i<count; i++) {
+		memcpy(buffer, Atari2HostAddr(address), secsize);
+		address += secsize;
+		if (::write(device, buffer, secsize) != secsize) return false;
+	}
+	return true;
+}
 void ACSIFDC::fdc_exec_command()
 {
 	static int dir=1,motor=1;
 	int actual_side, d;
-	memptr address;
 	long offset;
-	long count;
-	uint8 *buffer;
 
-	address = DMAaddr;
-	buffer = Atari2HostAddr(address);
 	int snd_porta = getYAMAHA()->getFloppyStat();
-	D(bug("FDC DMA virtual address = %06x, physical = %08x, snd = %d", address, buffer, snd_porta));
+	D(bug("FDC DMA address = %06x, snd = %d", DMAaddr, snd_porta));
 	actual_side=(~snd_porta)&1;
 	d=(~snd_porta)&6;
 	switch(d)
@@ -426,14 +453,24 @@ void ACSIFDC::fdc_exec_command()
 	{
 		if (d>=0)
 		{
+			int record_not_found = 0;
 			offset=secsize
 				* (((spt*sides*head))
 				+ (spt * actual_side) + (fdc_sector-1));
+			// special hack for 'fixing' dma_scr in Linux where it's often = 20
+			int newscr = spt - fdc_sector + 1;
+			if (newscr < dma_scr && spt > 1) {
+				D(bug("FDC: Fixed SCR from %d to %d", dma_scr, newscr));
+				record_not_found = 1;
+				dma_scr = newscr;
+			}
 			switch(fdc_command & 0xf0)
 			{
 				case 0x80:
-					D(bug("\tFDC READ SECTOR %d to 0x%06lx", (secsize)?offset/secsize:-1/*dma_scr*/,address));
-
+					assert(dma_scr == 1); // otherwise the fallthrough will cause problems
+					// fallthrough
+				case 0x90:
+					D(bug("\tFDC READ SECTOR  %d to 0x%06lx", dma_scr, DMAaddr));
 #if REMOUNT_FLOPPY_ON_BOOTSECTOR_READING
 					// special hack for remounting physical floppy on
 					// bootsector access
@@ -443,71 +480,36 @@ void ACSIFDC::fdc_exec_command()
 						insert_floppy();
 					}
 #endif
-					count=dma_scr*secsize;
-					if (lseek(drive_fd, offset, SEEK_SET)>=0)
-					{
-						if (count==read(drive_fd, buffer, count))
-						{
-							address += count;
-							DMAaddr = address;
-							dma_scr=0;
-							dma_sr=1;
-							break;
-						}
-						else {
-							D(bug("read(%d, %d, %d) didn't return correct len.", drive_fd, buffer, count));
-						}
+					if (read_file(drive_fd, offset, DMAaddr, secsize, dma_scr)) {
+						DMAaddr += dma_scr*secsize;
+						dma_scr=0;
+						dma_sr=1;
+						if (record_not_found)
+							fdc_status |= 0x10;
+						break;
 					}
-					fdc_status |= 0x10;
-					dma_sr=1;
-					break;
-				case 0x90:
-					D(bug("\tFDC READ SECTOR M. %d to 0x%06lx", dma_scr,address));
-					count=dma_scr*secsize;
-					if (lseek(drive_fd, offset, SEEK_SET)>=0)
-					{
-						if (count==read(drive_fd, buffer, count))
-						{
-							address += count;
-							DMAaddr = address;
-							dma_scr=0;
-							dma_sr=1;
-							fdc_sector += dma_scr; /* *(512/secsize);*/
-							break;
-						}
+					else {
+						if (! panic_floppy_error++)
+							panicbug("Floppy read(%d, %06x, %d) failed.", drive_fd, DMAaddr, dma_scr);
 					}
 					fdc_status |= 0x10;
 					dma_sr=1;
 					break;
 				case 0xa0:
-					count=dma_scr*secsize;
-					if (lseek(drive_fd, offset, SEEK_SET)>=0)
-					{
-						if (count==write(drive_fd, buffer, count))
-						{
-							address += count;
-							DMAaddr = address;
-							dma_scr=0;
-							dma_sr=1;
-							break;
-						}
-					}
-					fdc_status |= 0x10;
-					dma_sr=1;
-					break;
+					assert(dma_scr == 1); // otherwise the fallthrough will cause problems
+					// fallthrough
 				case 0xb0:
-					count=dma_scr*secsize;
-					if (lseek(drive_fd, offset, SEEK_SET)>=0)
-					{
-						if (count==write(drive_fd, buffer, count))
-						{
-							address += count;
-							DMAaddr = address;
-							dma_scr=0;
-							dma_sr=1;
-							fdc_sector += dma_scr; /* *(512/secsize);*/
-							break;
-						}
+					if (write_file(drive_fd, offset, DMAaddr, secsize, dma_scr)) {
+						DMAaddr += dma_scr*secsize;
+						dma_scr=0;
+						dma_sr=1;
+						if (record_not_found)
+							fdc_status |= 0x10;
+						break;
+					}
+					else {
+						if (! panic_floppy_error++)
+							panicbug("Floppy write(%d, %06x, %d) failed.", drive_fd, DMAaddr, dma_scr);
 					}
 					fdc_status |= 0x10;
 					dma_sr=1;
@@ -531,3 +533,7 @@ void ACSIFDC::fdc_exec_command()
 	if (!(fdc_status & 1))
 		getMFP()->setGPIPbit(0x20, 0);
 }
+
+/*
+vim:ts=4:sw=4:
+*/
