@@ -1,7 +1,7 @@
 /*
  * hostscreen.cpp - host video routines
  *
- * Copyright (c) 2001-2003 STanda of ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2005 STanda of ARAnyM developer team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -29,6 +29,7 @@
 #include "hostscreen.h"
 #include "parameters.h"
 #include "gui-sdl/sdlgui.h"
+#include "main.h"
 
 #define DEBUG 0
 #include "debug.h"
@@ -83,7 +84,13 @@ HostScreen::HostScreen(void) {
 	// OpenGL stuff
 	SdlGlSurface=NULL;
 	SdlGlTexture=NULL;
+	dirty_rects=NULL;
+	dirty_w=dirty_h=0;
+	npot_texture=rect_texture=SDL_FALSE;
+	rect_target=GL_TEXTURE_2D;
 #endif /* ENABLE_OPENGL */
+
+	DisableOpenGLVdi();
 }
 
 HostScreen::~HostScreen(void) {
@@ -97,6 +104,11 @@ HostScreen::~HostScreen(void) {
 	// OpenGL stuff
 #ifdef ENABLE_OPENGL
 	if (bx_options.opengl.enabled) {
+		if (dirty_rects) {
+			delete dirty_rects;
+			dirty_rects=NULL;
+		}
+
 		if (mainSurface) {
 			SDL_FreeSurface(mainSurface);
 			mainSurface=NULL;
@@ -122,9 +134,33 @@ void HostScreen::makeSnapshot()
 void HostScreen::toggleFullScreen()
 {
 	bx_options.video.fullscreen = !bx_options.video.fullscreen;
-	SDL_WM_ToggleFullScreen(mainSurface);
+	sdl_videoparams ^= SDL_FULLSCREEN;
+	if(SDL_WM_ToggleFullScreen(mainSurface) == 0) {
+		// SDL_WM_ToggleFullScreen() did not work.
+		// We have to change video mode "by hand".
+		SDL_Surface *temp = SDL_ConvertSurface(mainSurface, mainSurface->format,
+		                                       mainSurface->flags);
+		if (temp == NULL)
+			bug("toggleFullScreen: Unable to save screen content.");
+
+		mainSurface = SDL_SetVideoMode(width, height, bpp, sdl_videoparams);
+		if (mainSurface == NULL)
+			bug("toggleFullScreen: Unable to set new video mode.");
+		if (mainSurface->format->BitsPerPixel <= 8)
+			SDL_SetColors(mainSurface, temp->format->palette->colors, 0,
+			              temp->format->palette->ncolors);
+
+		if (SDL_BlitSurface(temp, NULL, mainSurface, NULL) != 0)
+			bug("toggleFullScreen: Unable to restore screen content.");
+		SDL_FreeSurface(temp);
+#ifdef SDL_GUI
+		if (isGUIopen() == 0)
+			surf = mainSurface;
+#endif /* SDL_GUI */
+	}
 }
 
+#ifdef SDL_GUI
 void HostScreen::allocateBackgroundSurf()
 {
 	// allocate new background video surface
@@ -185,7 +221,6 @@ void HostScreen::restoreBackground()
 		D(bug("video surface restored"));
 	}
 }
-
 void HostScreen::blendBackgrounds()
 {
 	if (backgroundSurf != NULL) {
@@ -199,6 +234,7 @@ void HostScreen::blendBackgrounds()
 		update(true);
 	}
 }
+#endif /* SDL_GUI */
 
 int HostScreen::selectVideoMode(SDL_Rect **modes, uint32 *width, uint32 *height)
 {
@@ -298,100 +334,191 @@ void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 
 #ifdef ENABLE_OPENGL
 	if (bx_options.opengl.enabled) {
-		GLint MaxTextureSize;
-		int filtering;		
+		int filtering, i, gl_bpp[4]={0,16,24,32};
 
-		/* Set video mode if not already done */
-		if (SdlGlSurface==NULL) {
-			sdl_videoparams |= SDL_OPENGL;
+		sdl_videoparams |= SDL_OPENGL;
 
-/*			SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, bx_options.opengl.bpp);*/
-			SdlGlSurface = SDL_SetVideoMode(bx_options.opengl.width, bx_options.opengl.height, 0 /*bx_options.opengl.bpp*/, sdl_videoparams);
+		/* Setup at least 15 bits true colour OpenGL context */
+		SDL_GL_SetAttribute(SDL_GL_RED_SIZE,5);
+		SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE,5);
+		SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,5);
+		SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE,15);
 
-			glViewport(0, 0, bx_options.opengl.width, bx_options.opengl.height);
-
-			/* Projection matrix */
-			glMatrixMode(GL_PROJECTION);
-			glLoadIdentity();
-			gluOrtho2D(0.0, bx_options.opengl.width, bx_options.opengl.height, 0.0);
-
-			/* Texture matrix */
-			glMatrixMode(GL_TEXTURE);
-			glLoadIdentity();
-
-			/* Model view matrix */
-			glMatrixMode(GL_MODELVIEW);
-			glLoadIdentity();
-
-			/* Enable texturing */
-			glEnable(GL_TEXTURE_2D);
-			{
-				glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+		for (i=0;i<4;i++) {
+			SdlGlSurface = SDL_SetVideoMode(width, height, gl_bpp[i], sdl_videoparams);
+			if (SdlGlSurface) {
+				break;
 			}
-			D(bug("gl: video mode set"));
 		}
+		if (!SdlGlSurface) {
+			fprintf(stderr,"Can not setup %dx%d OpenGL video mode\n",width,height);
+			QuitEmulator();
+		}
+		this->width = width = SdlGlSurface->w;
+		this->height = height = SdlGlSurface->h;
+		this->bpp = bpp = 32;	/* bpp of texture that will be used */
 
-		/* Create a surface for Aranym */
+		glViewport(0, 0, width, height);
+
+		/* Projection matrix */
+		glMatrixMode(GL_PROJECTION);
+		glLoadIdentity();
+		glOrtho(0.0, width, height, 0.0, -1.0, 1.0);
+
+		/* Texture matrix */
+		glMatrixMode(GL_TEXTURE);
+		glLoadIdentity();
+
+		/* Model view matrix */
+		glMatrixMode(GL_MODELVIEW);
+		glLoadIdentity();
+		glTranslatef(0.375, 0.375, 0.0);
+
+		/* Setup texturing mode */
+		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+
+		/* Delete previous stuff */
 		if (mainSurface) {
 			SDL_FreeSurface(mainSurface);
+			mainSurface=NULL;
 		}
-		
-		this->bpp = bpp = 32;
 
-		/* Create a texture */
 		if (SdlGlTexture) {
 			glDeleteTextures(1, &SdlGlTexObj);
 		}
 
-		glGetIntegerv(GL_MAX_TEXTURE_SIZE, &MaxTextureSize);
+		/* Full screen OpenGL rendering ? */
+		if (!OpenGLVdi) {
+			GLint MaxTextureSize;
+			char *extensions;
 
-		D(bug("gl: need at least a %dx%d texture",width,height));
-		D(bug("gl: texture is at most a %dx%d texture",MaxTextureSize,MaxTextureSize));
+			rect_target=GL_TEXTURE_2D;
+			npot_texture = rect_texture = SDL_FALSE;
+			glGetIntegerv(GL_MAX_TEXTURE_SIZE, &MaxTextureSize);
 
-		/* Calculate the smallest needed texture */
-		SdlGlTextureWidth = SdlGlTextureHeight = MaxTextureSize;
-		if (width<SdlGlTextureWidth) {
-			while (((SdlGlTextureWidth>>1)>width) && (SdlGlTextureWidth>64)) {
-				SdlGlTextureWidth>>=1;
+			extensions = (char *)glGetString(GL_EXTENSIONS);
+
+			/* Check texture rectangle extensions */
+
+#if defined(GL_NV_texture_rectangle)
+			if (strstr(extensions, "GL_NV_texture_rectangle")) {
+				rect_texture = SDL_TRUE;
+				rect_target=GL_TEXTURE_RECTANGLE_NV;
+				glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_NV, &MaxTextureSize);
 			}
-		}
-		if (height<SdlGlTextureHeight) {
-			while (((SdlGlTextureHeight>>1)>height) && (SdlGlTextureHeight>64)) {
-				SdlGlTextureHeight>>=1;
+#endif
+#if defined(GL_EXT_texture_rectangle)
+			if (strstr(extensions, "GL_EXT_texture_rectangle")) {
+				rect_texture = SDL_TRUE;
+				rect_target=GL_TEXTURE_RECTANGLE_EXT;
+				glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_EXT, &MaxTextureSize);
 			}
+#endif
+#if defined(GL_ARB_texture_rectangle)
+			if (strstr(extensions, "GL_ARB_texture_rectangle")) {
+				rect_texture = SDL_TRUE;
+				rect_target=GL_TEXTURE_RECTANGLE_ARB;
+				glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_ARB, &MaxTextureSize);
+			}
+#endif
+
+			/* Check non power of two texture extension */
+			npot_texture = rect_texture;
+			if (strstr(extensions, "GL_ARB_texture_non_power_of_two")) {
+				npot_texture=SDL_TRUE;
+				rect_texture=SDL_FALSE;
+				rect_target=GL_TEXTURE_2D;
+				glGetIntegerv(GL_MAX_TEXTURE_SIZE, &MaxTextureSize);
+			}
+
+			SdlGlTextureWidth = SdlGlTextureHeight = MaxTextureSize;
+			D(bug("gl: need at least a %dx%d texture",width,height));
+			D(bug("gl: texture is at most a %dx%d texture",MaxTextureSize,MaxTextureSize));
+
+			if (!npot_texture) {
+				/* Calculate the smallest needed texture */
+				if (width<SdlGlTextureWidth) {
+					while (((SdlGlTextureWidth>>1)>width) && (SdlGlTextureWidth>64)) {
+						SdlGlTextureWidth>>=1;
+					}
+				}
+				if (height<SdlGlTextureHeight) {
+					while (((SdlGlTextureHeight>>1)>height) && (SdlGlTextureHeight>64)) {
+						SdlGlTextureHeight>>=1;
+					}
+				}
+			} else {
+				if (width<SdlGlTextureWidth) {
+					if (width>64) {
+						SdlGlTextureWidth=width;
+					} else {
+						SdlGlTextureWidth=64;
+					}
+				}
+				if (height<SdlGlTextureHeight) {
+					if (height>64) {
+						SdlGlTextureHeight=height;
+					} else {
+						SdlGlTextureHeight=64;
+					}
+				}
+			}
+			D(bug("gl: texture will be %dx%d texture",SdlGlTextureWidth,SdlGlTextureHeight));
 		}
 
-		D(bug("gl: texture will be %dx%d texture",SdlGlTextureWidth,SdlGlTextureHeight));
-
-		mainSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, SdlGlTextureWidth,SdlGlTextureHeight,bpp, 255<<16,255<<8,255,255<<24);
+		mainSurface = SDL_CreateRGBSurface(SDL_SWSURFACE,
+			SdlGlTextureWidth,SdlGlTextureHeight,32,
+#if SDL_BYTEORDER == SDL_LIL_ENDIAN
+			255<<16,255<<8,255,255<<24	/* GL_BGRA little endian */
+#else
+			255<<8,255<<16,255<<24,255	/* GL_BGRA big endian */
+#endif
+		);
+		if (!mainSurface) {
+			fprintf(stderr,"Can not create %dx%dx%d texture\n",SdlGlTextureWidth,SdlGlTextureHeight,bpp);
+			QuitEmulator();
+		}
 		SdlGlTexture = (uint8 *) (mainSurface->pixels);
 
-		glGenTextures(1, &SdlGlTexObj);
-		glBindTexture(GL_TEXTURE_2D, SdlGlTexObj);
+		if (!OpenGLVdi) {
+			glGenTextures(1, &SdlGlTexObj);
+			glBindTexture(GL_TEXTURE_2D, SdlGlTexObj);
 
-		filtering = GL_NEAREST;		
-		if (bx_options.opengl.filtered) {
-			filtering = GL_LINEAR;
-		}
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filtering); // scale when image bigger than texture
-		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filtering); // scale when image smaller than texture
+			filtering = GL_NEAREST;		
+			if (bx_options.opengl.filtered) {
+				filtering = GL_LINEAR;
+			}
+			glTexParameteri(rect_target, GL_TEXTURE_MAG_FILTER, filtering); // scale when image bigger than texture
+			glTexParameteri(rect_target, GL_TEXTURE_MIN_FILTER, filtering); // scale when image smaller than texture
 
-		glTexImage2D(GL_TEXTURE_2D, 0, 4, SdlGlTextureWidth, SdlGlTextureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, SdlGlTexture);
+			glTexImage2D(rect_target, 0, GL_RGBA, SdlGlTextureWidth, SdlGlTextureHeight, 0, GL_BGRA, GL_UNSIGNED_BYTE, SdlGlTexture);
 
-		D(bug("gl: texture created"));
+			D(bug("gl: texture created"));
 
-		/* Activate autozoom if texture smaller than screen */
-		if ((width>SdlGlTextureWidth) || (height>SdlGlTextureHeight)) {
-			this->width = width = SdlGlTextureWidth;
-			this->height = height = SdlGlTextureHeight;
-			bx_options.autozoom.enabled = true;
-			bx_options.autozoom.integercoefs = false;
+			/* Activate autozoom if texture smaller than screen */
+			SdlGlWidth = width;
+			SdlGlHeight = height;
+			if ((width>SdlGlTextureWidth) || (height>SdlGlTextureHeight)) {
+				this->width = width = SdlGlTextureWidth;
+				this->height = height = SdlGlTextureHeight;
+				bx_options.autozoom.enabled = true;
+				bx_options.autozoom.integercoefs = false;
 
-			D(bug("gl: autozoom enabled"));
-		} else {
-			bx_options.autozoom.enabled = false;
-			bx_options.autozoom.integercoefs = false;
-			D(bug("gl: autozoom disabled"));
+				D(bug("gl: autozoom enabled"));
+			} else {
+				bx_options.autozoom.enabled = false;
+				bx_options.autozoom.integercoefs = false;
+				D(bug("gl: autozoom disabled"));
+			}
+
+			/* Create dirty rectangles list */
+			if (dirty_rects)
+				delete dirty_rects;
+		
+			dirty_w=((width|15)+1)>>4;
+			dirty_h=((height|15)+1)>>4;
+			dirty_rects=new SDL_bool[dirty_w*dirty_h];
+			memset(dirty_rects,SDL_FALSE,sizeof(SDL_bool)*dirty_w*dirty_h);
 		}
 	}
 	else
@@ -400,6 +527,7 @@ void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 		mainSurface = SDL_SetVideoMode(width, height, bpp, sdl_videoparams);
 	}
 
+#ifdef SDL_GUI
 	if (isGUIopen()) {
 		freeBackgroundSurf();
 		allocateBackgroundSurf();
@@ -410,7 +538,9 @@ void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 		event.user.code = SDL_USEREVENT; // misused this code for signalizing the resolution change. Did that because I knew the code was unique (needed something distinguishable from keyboard and mouse codes that are sent by the same event name from the input checking thread)
 		SDL_PeepEvents(&event, 1, SDL_ADDEVENT, SDL_EVENTMASK(SDL_USEREVENT));
 	}
-	else {
+	else
+#endif /* SDL_GUI */
+	{
 		surf = mainSurface;
 	}
 
@@ -444,7 +574,7 @@ void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 			surf->format->Rloss, surf->format->Gloss, surf->format->Bloss));
 }
 
-
+/*
 extern "C" {
 	static void getBinary( uint16 data, char *buffer ) {
 		for( uint16 i=0; i<=15; i++ ) {
@@ -454,7 +584,7 @@ extern "C" {
 		buffer[16]='\0';
 	}
 }
-
+*/
 
 
 inline void HostScreen::putPixel( int16 x, int16 y, uint16 pattern, uint32 fgColor, uint32 bgColor, uint16 logOp )
@@ -1245,9 +1375,220 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 	}  // switch
 }
 
+void HostScreen::update( int32 x, int32 y, int32 w, int32 h, bool forced )
+{
+	if ( !forced && !doUpdate ) // the HW surface is available
+		return;
+
+	//	SDL_UpdateRect(SDL_GetVideoSurface(), 0, 0, width, height);
+	// SDL_UpdateRect(surf, x, y, w, h);
+	SDL_UpdateRect(mainSurface, x, y, w, h);
+
+#ifdef ENABLE_OPENGL
+	if (!OpenGLVdi && bx_options.opengl.enabled && dirty_rects) {
+		/* Mark dirty rectangles */
+		int x1,y1,x2,y2, i,j;
+		
+		x1=x>>4;
+		y1=y>>4;
+		x2=x+w;
+		if (x2&15) x2=(x2|15)+1;
+		x2>>=4;
+		y2=y+h;
+		if (y2&15) y2=(y2|15)+1;
+		y2>>=4;
+
+		for (j=y1;j<y2;j++) {
+			for (i=x1;i<x2;i++) {
+				dirty_rects[i+(j*dirty_w)]=SDL_TRUE;
+			}
+		}
+	}
+#endif
+}
+
+void HostScreen::update( bool forced )
+{
+	update( 0, 0, width, height, forced );
+}
+
+void HostScreen::update()
+{
+	update( 0, 0, width, height, false );
+}
+
+void HostScreen::OpenGLUpdate(void)
+{
+#ifdef ENABLE_OPENGL
+	GLfloat tex_width, tex_height;
+
+	if (OpenGLVdi) {
+		return;
+	}
+
+	glEnable(rect_target);
+	glBindTexture(rect_target, SdlGlTexObj);
+
+	/* Update the texture */
+	{
+		int x,y;
+
+		for (y=0;y<dirty_h;y++) {
+			SDL_bool update_line=SDL_FALSE;
+			for (x=0;x<dirty_w;x++) {
+				if (dirty_rects[y*dirty_w+x]==SDL_TRUE) {
+					update_line=SDL_TRUE;
+					break;
+				}
+			}
+			if (update_line) {
+				glTexSubImage2D(rect_target, 0,
+					 0, y<<4,
+					 SdlGlTextureWidth, 16,
+					 GL_BGRA, GL_UNSIGNED_BYTE,
+					 SdlGlTexture + (y<<4)*SdlGlTextureWidth*4
+				);
+			}
+		}
+	}
+	memset(dirty_rects,SDL_FALSE,sizeof(SDL_bool)*dirty_w*dirty_h);
+
+	/* Render the textured quad */
+	tex_width = ((GLfloat)width)/((GLfloat)SdlGlTextureWidth);
+	tex_height = ((GLfloat)height)/((GLfloat)SdlGlTextureHeight);
+	if (rect_target!=GL_TEXTURE_2D) {
+		tex_width = (GLfloat)width;
+		tex_height = (GLfloat)height;
+	}
+
+	glBegin(GL_QUADS);
+		glTexCoord2f( 0.0, 0.0 );
+		glVertex2i( 0, 0);
+
+		glTexCoord2f( tex_width, 0.0 );
+		glVertex2i( SdlGlWidth, 0);
+
+		glTexCoord2f( tex_width, tex_height);
+		glVertex2i( SdlGlWidth, SdlGlHeight);
+
+		glTexCoord2f( 0.0, tex_height);
+		glVertex2i( 0, SdlGlHeight);
+	glEnd();
+
+	glDisable(rect_target);
+#endif
+}
+
+uint32 HostScreen::getBitsPerPixel(void)
+{
+	return surf->format->BitsPerPixel;
+}
+
+void HostScreen::EnableOpenGLVdi(void)
+{
+	OpenGLVdi = SDL_TRUE;
+}
+
+void HostScreen::DisableOpenGLVdi(void)
+{
+	OpenGLVdi = SDL_FALSE;
+}
 
 /*
  * $Log: hostscreen.cpp,v $
+ * Revision 1.78  2005/12/26 20:35:44  pmandin
+ * Remove unneeded inclusion of glext.h, use SDL included one
+ *
+ * Revision 1.77  2005/11/14 00:38:28  stefanq
+ * - nfvdi_opengl.h: Changed include of GL/glu.h to OpenGL/glu.h for darwin.
+ * - nfvdi_opengl.cpp: Changed include of GL/glu.h to OpenGL/glu.h for darwin.
+ * - hostscreen.cpp: Changed include of GL/glu.h to OpenGL/glu.h and GL/glext.h
+ *   to OpenGL/glext.h for darwin.
+ *
+ * Revision 1.76  2005/08/05 08:53:24  johan
+ * Removed unnecessary dependency on glu function.
+ *
+ * Revision 1.75  2005/06/21 12:50:27  pmandin
+ * Small fixes for OpenGL rendering
+ *
+ * Revision 1.74  2005/06/16 13:26:47  milan
+ * Disabled for non-gui version
+ *
+ * Revision 1.73  2005/06/14 16:45:04  pmandin
+ * Use rectangle texture when available
+ *
+ * Revision 1.72  2005/06/14 15:11:43  pmandin
+ * Use non power of two texture when available
+ *
+ * Revision 1.71  2005/06/13 12:58:05  pmandin
+ * Correctly setup shadow texture when smaller than asked
+ *
+ * Revision 1.70  2005/06/13 12:12:41  pmandin
+ * Disable texturing when not needed
+ *
+ * Revision 1.69  2005/06/12 15:32:00  pmandin
+ * OpenGL video mode was locked to 640x480
+ *
+ * Revision 1.68  2005/06/12 15:03:22  pmandin
+ * Remove width and height parameters in OpenGL section
+ *
+ * Revision 1.67  2005/06/11 20:13:14  pmandin
+ * Remove getComponent stuff
+ *
+ * Revision 1.66  2005/06/09 19:50:16  pmandin
+ * Fixing getComponent function
+ *
+ * Revision 1.65  2005/06/08 19:33:02  pmandin
+ * Add getComponent() function
+ *
+ * Revision 1.64  2005/06/08 18:14:59  pmandin
+ * Update texture using a dirty rectangles list
+ *
+ * Revision 1.63  2005/06/08 15:50:44  pmandin
+ * Correct endianness of GL_RGBA shadow surface
+ *
+ * Revision 1.62  2005/06/07 21:30:36  johan
+ * fix for integer coordinates according to MicroSoft tip
+ *
+ * Revision 1.61  2005/06/06 08:06:49  pmandin
+ * Revert back off by 1 stuff
+ *
+ * Revision 1.60  2005/05/24 12:48:43  pmandin
+ * OpenGL rendering off by 1 unit
+ *
+ * Revision 1.59  2005/05/12 11:29:25  pmandin
+ * Disable texturing with OpenGL NFVDI
+ *
+ * Revision 1.58  2005/05/12 07:55:08  pmandin
+ * Keep OpenGL related stuff in hostscreen and nfvdi classes
+ *
+ * Revision 1.57  2005/05/11 20:19:26  pmandin
+ * Add NF vdi OpenGL renderer
+ *
+ * Revision 1.56  2005/05/11 16:16:24  pmandin
+ * Confused by bpp (bits per pixel) and bpp (bytes per pixel)
+ *
+ * Revision 1.55  2005/05/10 16:57:54  pmandin
+ * Forgot some #IFDEF
+ *
+ * Revision 1.54  2005/05/10 10:37:44  pmandin
+ * Faster OpenGL rendering
+ *
+ * Revision 1.53  2005/04/22 21:57:16  xavier
+ * Remove debug code left by mistake.
+ *
+ * Revision 1.52  2005/04/22 21:52:36  xavier
+ * Switch between fullscreen and windowed mode on Microsoft Windows
+ *
+ * Revision 1.51  2005/01/22 16:39:48  joy
+ * --disable-gui now disables all SDL GUI related code
+ *
+ * Revision 1.50  2005/01/12 14:35:16  joy
+ * more unused variables hidden
+ *
+ * Revision 1.49  2004/12/11 09:55:52  pmandin
+ * Move update function from .h to .cpp, mandatory for NFOSMesa
+ *
  * Revision 1.48  2004/10/31 23:15:56  pmandin
  * Do not need to enable autozoom to force screen size
  *
@@ -1416,3 +1757,7 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
  *
  *
  */
+
+/*
+vim:ts=4:sw=4:
+*/
