@@ -124,11 +124,7 @@ uint8 IKBD::ReadData()
 			/* Queue empty */
 
 			/* Update MFP GPIP */
-			// uint8 x = ReadAtariInt8(0xfffa01);
-			uint8 x = getMFP()->handleRead(0xfffa01);
-			x |= 0x10;
-			// WriteAtariInt8(0xfffa01, x);
-			getMFP()->handleWrite(0xfffa01, x);
+			getMFP()->setGPIPbit(0x10, 0x10);
 
 			sr &= ~((1<<ACIA_SR_INTERRUPT)|(1<<ACIA_SR_RXFULL));
 
@@ -268,6 +264,7 @@ void IKBD::WriteData(uint8 value)
 					if (outbuffer[1]==0x01) {
 						reset();
 						outwrite = 0;
+						send(0xf1); // if all's well code 0xF1 is returned, else the break codes of all keys making contact 
 					}
 					break;
 			}
@@ -474,6 +471,29 @@ void IKBD::SendJoystickAxis(int numjoy, int numaxis, int value)
 	}
 }
 
+void IKBD::SendJoystickHat(int numjoy, int value)
+{
+	uint8 newjoy_state;
+
+	if (!joy_enabled[numjoy])
+		return;
+
+	/* Keep current button press */
+	newjoy_state = joy_state[numjoy] & (1<<IKBD_JOY_FIRE);
+
+	if (value & SDL_HAT_LEFT) newjoy_state |= 1<<IKBD_JOY_LEFT;
+	if (value & SDL_HAT_RIGHT) newjoy_state |= 1<<IKBD_JOY_RIGHT;
+	if (value & SDL_HAT_UP) newjoy_state |= 1<<IKBD_JOY_UP;
+	if (value & SDL_HAT_DOWN) newjoy_state |= 1<<IKBD_JOY_DOWN;
+
+	if (joy_state[numjoy] != newjoy_state) {
+		joy_state[numjoy] = newjoy_state;
+		intype = IKBD_PACKET_JOYSTICK;
+		send(0xfe | numjoy);
+		send(joy_state[numjoy]);
+	}
+}
+
 void IKBD::SendJoystickButton(int numjoy, int pressed)
 {
 	uint8 newjoy_state;
@@ -550,7 +570,6 @@ void IKBD::send(uint8 value)
 void IKBD::ThrowInterrupt(void)
 {
 	/* Check MFP IER */
-	// if ((HWget_b(0xfffa09) & 0x40)==0)
 	if ((getMFP()->handleRead(0xfffa09) & 0x40)==0)
 		return;
 
