@@ -24,23 +24,11 @@
 #ifndef _HOSTSCREEN_H
 #define _HOSTSCREEN_H
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
-#ifdef HAVE_NEW_HEADERS
 # include <cstdlib>
 # include <cstring>
-#else
-# include <stdlib.h>
-# include <string.h>
-#endif
 
 #include <SDL.h>
 #include <SDL_thread.h>
-#ifdef ENABLE_OPENGL
-#include <SDL_opengl.h>
-#endif
 
 #include "parameters.h"
 
@@ -87,13 +75,19 @@ class HostScreen {
 	void searchVideoMode( uint32 *width, uint32 *height, uint32 *bpp );
 
 	// OpenGL stuff
-#ifdef ENABLE_OPENGL
 	SDL_Surface *SdlGlSurface;
-	GLuint SdlGlTexObj;
-	GLuint SdlGlTextureWidth;
-	GLuint SdlGlTextureHeight;
+	unsigned int SdlGlTexObj;
+	unsigned int SdlGlTextureWidth;
+	unsigned int SdlGlTextureHeight;
 	uint8 *SdlGlTexture;
-#endif /* ENABLE_OPENGL */
+	int SdlGlWidth, SdlGlHeight;
+	SDL_bool npot_texture;		/* Use non power of two texture ? */
+	SDL_bool rect_texture;		/* Use rectangle texture ? */
+	int rect_target;
+	SDL_bool OpenGLVdi;			/* Using NF OpenGL VDI renderer ? */
+	SDL_bool *dirty_rects;		/* Dirty rectangles list */
+	int dirty_w,dirty_h;
+
   public:
 	SDL_mutex   *screenLock;
 	uint32 sdl_videoparams;
@@ -133,10 +127,15 @@ class HostScreen {
 	inline void	  renderEnd();
 
 	// the w, h should be width & height (but C++ complains -> 'if's in the implementation)
-	inline void	  update( int32 x, int32 y, int32 w, int32 h, bool forced = false );
-	inline void	  update( bool forced );
-	inline void	  update();
+	void update( int32 x, int32 y, int32 w, int32 h, bool forced = false );
+	void update( bool forced );
+	void update();
 
+	void OpenGLUpdate(void);	/* Full screen update with NF software VDI */
+	void EnableOpenGLVdi(void);
+	void DisableOpenGLVdi(void);
+
+#ifdef SDL_GUI
 	// GUI
 	void openGUI();
 	void closeGUI();
@@ -146,8 +145,10 @@ class HostScreen {
 	void restoreBackground();
 	void blendBackgrounds();
 	SDL_Surface *getPhysicalSurface() { return mainSurface; }
+#endif
 
-	uint32 getBpp();
+	uint32 getBpp();	/* Bytes per pixel */
+	uint32 getBitsPerPixel(void);
 	uint32 getPitch();
 	uint32 getWidth();
 	uint32 getHeight();
@@ -233,9 +234,22 @@ inline uint32 HostScreen::gfxGetPixel( int16 x, int16 y )
 	return 0;	// should never happen
 }
 
-
-
-inline uint32 HostScreen::getBpp() {
+inline uint32 HostScreen::getBpp()
+{
+#ifdef ENABLE_OPENGL
+	if (bx_options.opengl.enabled) {
+		switch(bpp) {
+			case 15:
+			case 16:
+				return 2;
+			case 24:
+				return 3;
+			case 32:
+				return 4;
+		}
+		return 1;
+	}
+#endif
 	return surf->format->BytesPerPixel;
 }
 
@@ -303,55 +317,6 @@ inline void HostScreen::renderEnd() {
 		SDL_UnlockSurface(surf);
 }
 
-
-inline void HostScreen::update( int32 x, int32 y, int32 w, int32 h, bool forced )
-{
-	if ( !forced && !doUpdate ) // the HW surface is available
-		return;
-
-	//	SDL_UpdateRect(SDL_GetVideoSurface(), 0, 0, width, height);
-	// SDL_UpdateRect(surf, x, y, w, h);
-#ifdef ENABLE_OPENGL
-	if (bx_options.opengl.enabled) {
-		/* Update the texture */
-		glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, SdlGlTextureWidth, SdlGlTextureHeight, GL_RGBA, GL_UNSIGNED_BYTE, SdlGlTexture);
-
-		/* Render the textured quad */
-		glBegin(GL_QUADS);
-			glTexCoord2f( 0.0, 0.0 );
-			glVertex2i( 0, 0);
-
-			glTexCoord2f( (GLfloat)(((GLfloat)width)/((GLfloat)SdlGlTextureWidth)), 0.0 );
-			glVertex2i( bx_options.opengl.width, 0);
-
-			glTexCoord2f( (GLfloat)(((GLfloat)width)/((GLfloat)SdlGlTextureWidth)), (GLfloat)(((GLfloat)height)/((GLfloat)SdlGlTextureHeight)) );
-			glVertex2i( bx_options.opengl.width, bx_options.opengl.height);
-
-			glTexCoord2f( 0.0, (GLfloat)(((GLfloat)height)/((GLfloat)SdlGlTextureHeight)) );
-			glVertex2i( 0, bx_options.opengl.height);
-		glEnd();
-
-		SDL_GL_SwapBuffers();
-	}
-	else
-#endif	/* ENABLE_OPENGL */
-	{
-		SDL_UpdateRect(mainSurface, x, y, w, h);
-	}
-}
-
-inline void HostScreen::update( bool forced )
-{
-	update( 0, 0, width, height, forced );
-}
-
-
-inline void HostScreen::update()
-{
-	update( 0, 0, width, height, false );
-}
-
-
 inline uint32 HostScreen::getPixel( int16 x, int16 y ) {
 	if ( x < 0 || x >= (int32)width || y < 0 || y >= (int32)height )
 		return 0;
@@ -457,6 +422,51 @@ inline void HostScreen::bitplaneToChunky( uint16 *atariBitplaneData, uint16 bpp,
 
 /*
  * $Log: hostscreen.h,v $
+ * Revision 1.64  2005/06/14 16:45:04  pmandin
+ * Use rectangle texture when available
+ *
+ * Revision 1.63  2005/06/14 15:11:43  pmandin
+ * Use non power of two texture when available
+ *
+ * Revision 1.62  2005/06/13 12:58:05  pmandin
+ * Correctly setup shadow texture when smaller than asked
+ *
+ * Revision 1.61  2005/06/11 20:13:14  pmandin
+ * Remove getComponent stuff
+ *
+ * Revision 1.60  2005/06/09 19:50:16  pmandin
+ * Fixing getComponent function
+ *
+ * Revision 1.59  2005/06/08 19:33:02  pmandin
+ * Add getComponent() function
+ *
+ * Revision 1.58  2005/06/08 18:14:59  pmandin
+ * Update texture using a dirty rectangles list
+ *
+ * Revision 1.57  2005/05/12 07:55:08  pmandin
+ * Keep OpenGL related stuff in hostscreen and nfvdi classes
+ *
+ * Revision 1.56  2005/05/11 20:19:26  pmandin
+ * Add NF vdi OpenGL renderer
+ *
+ * Revision 1.55  2005/05/11 16:16:24  pmandin
+ * Confused by bpp (bits per pixel) and bpp (bytes per pixel)
+ *
+ * Revision 1.53  2005/05/10 21:03:40  pmandin
+ * Added getBpp function to FVDI driver
+ *
+ * Revision 1.52  2005/05/10 10:37:44  pmandin
+ * Faster OpenGL rendering
+ *
+ * Revision 1.51  2005/05/05 22:47:02  milan
+ * support of old headers removed
+ *
+ * Revision 1.50  2005/01/22 16:39:48  joy
+ * --disable-gui now disables all SDL GUI related code
+ *
+ * Revision 1.49  2004/12/11 09:55:52  pmandin
+ * Move update function from .h to .cpp, mandatory for NFOSMesa
+ *
  * Revision 1.48  2004/04/26 07:25:59  standa
  * GPL header & comment adjustments.
  *
