@@ -1,4 +1,29 @@
-/* MJ 2001 */
+/*
+ * parameters.h - parameter init/load/save code - header file
+ *
+ * Copyright (c) 2001-2005 ARAnyM developer team (see AUTHORS)
+ *
+ * Authors:
+ *  MJ		Milan Jurik
+ *  Joy		Petr Stehlik
+ * 
+ * This file is part of the ARAnyM project which builds a new and powerful
+ * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
+ *
+ * ARAnyM is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ARAnyM is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ARAnyM; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
 
 #ifndef PARAMETERS_H
 #define PARAMETERS_H
@@ -6,26 +31,29 @@
 #include "sysdeps.h"
 #include "version.h"
 #include "cfgopts.h"
+#include <SDL_keyboard.h>
 
-#ifdef HAVE_NEW_HEADERS
 # include <cassert>
 # include <cstdio>
-#else
-# include <assert.h>
-# include <stdio.h>
-#endif
 
 #ifndef ARANYMHOME
-# define ARANYMHOME		".aranym"
+# define ARANYMHOME	".aranym"
+#endif
+#ifndef ARANYMCONFIG
+# define ARANYMCONFIG	"config"
+#endif
+#ifndef ARANYMNVRAM
+# define ARANYMNVRAM	"nvram"
+#endif
+#ifndef ARANYMKEYMAP
+# define ARANYMKEYMAP	"keymap"
 #endif
-#define ARANYMCONFIG	"config"
-#define ARANYMNVRAM		"nvram"
-#define ARANYMKEYMAP	"keymap"
 #ifndef DIRSEPARATOR
 # define DIRSEPARATOR	"/"
 #endif
 
 #define BX_MAX_ATA_CHANNEL 1
+#define MAX_ETH		4
 
 enum geo_type {
 	geoCylinders,
@@ -73,18 +101,11 @@ typedef struct {
   bool byteswap;
   bool readonly;
   char path[512];
-  char partID[4];
+  char partID[3+1];	// 3 chars + EOS ( = '\0')
 } bx_scsidevice_options_t;
- /* 
-typedef struct {
-  char *path;
-  unsigned long address;
-  } bx_rom_options_t;
-*/
 
 // TOS options
 typedef struct {
-  bool redirect_PRT;
   bool redirect_CON;
   long cookie_mch;
 } bx_tos_options_t;
@@ -107,8 +128,7 @@ typedef struct {
 typedef struct {
   bool jit;
   bool jitfpu;
-  bool tunealign;
-  bool tunenop;
+  bool jitinline;
   uint32 jitcachesize;
   uint32 jitlazyflush;
   char jitblacklist[512];
@@ -117,9 +137,6 @@ typedef struct {
 // OpenGL options
 typedef struct {
   bool enabled;
-  uint32 width;
-  uint32 height;
-  uint32 bpp;
   bool filtered;
 } bx_opengl_options_t;
 
@@ -167,39 +184,70 @@ typedef struct {
   uint32 height;
 } bx_autozoom_options_t;
 
+// NfOSMesa options
+typedef struct {
+	uint32 channel_size;    /* If using libOSMesa[16|32], size of channel */
+	char libgl[256];		/* Pathname to libGL */
+	char libosmesa[256];	/* Pathname to libOSMesa */
+} bx_nfosmesa_options_t;
+
+// NatFeats options
+typedef struct {
+	char cdrom_driver[256];	/* CD-ROM driver */
+	char vdi_driver[256];	/* VDI driver */
+} bx_natfeat_options_t;
+
+// NFvdi options
+typedef struct {
+	bool use_host_mouse_cursor; /* Use host mouse cursor */
+} bx_nfvdi_options_t;
+
+// Parallel port options
+typedef struct {
+	char type[256];
+	char file[256];
+	char parport[256];
+} bx_parallel_options_t;
+
 // Keyboard and mouse
 typedef struct {
-  bool wheel_eiffel;		// eiffel compatible scancodes for mouse wheel
+  bool wheel_eiffel;	// eiffel compatible scancodes for mouse wheel
+  bool altgr;			// real AltGr like on Milan
 } bx_ikbd_options_t;
 
-/*
+// Hotkeys
 typedef struct {
-  char      *path;
-  bool   cmosImage;
-  unsigned int time0;
-  } bx_cmos_options;
-*/
+	SDL_keysym	setup;
+	SDL_keysym	quit;
+	SDL_keysym	reboot;
+	SDL_keysym	debug;
+	SDL_keysym	ungrab;
+	SDL_keysym	fullscreen;
+	SDL_keysym	screenshot;
+} bx_hotkeys_t;
+
+#ifdef GDBSTUB
+typedef struct {
+  memptr text_base;
+  memptr data_base;
+  memptr bss_base;
+} bx_gdbstub_t;
+#endif
+
+#define DISKS	8
 
 // Options 
 typedef struct {
   bx_floppy_options_t	floppy;
   bx_atadevice_options_t atadevice[BX_MAX_ATA_CHANNEL][2];
-  bx_scsidevice_options_t	disk0;
-  bx_scsidevice_options_t	disk1;
-  bx_scsidevice_options_t	disk2;
-  bx_scsidevice_options_t	disk3;
-  bx_scsidevice_options_t	disk4;
-  bx_scsidevice_options_t	disk5;
-  bx_scsidevice_options_t	disk6;
-  bx_scsidevice_options_t	disk7;
+  bx_scsidevice_options_t	disks[DISKS];
   bx_aranymfs_options_t	aranymfs[ 'Z'-'A'+1 ];
-//  bx_cookies_options_t cookies;
   bx_video_options_t	video;
   bx_tos_options_t	tos;
   bx_startup_options_t	startup;
   bx_jit_options_t	jit;
   bx_opengl_options_t	opengl;
-  bx_ethernet_options_t ethernet;
+  bx_ethernet_options_t ethernet[MAX_ETH];
   bx_lilo_options_t		lilo;
   bx_midi_options_t		midi;
   bx_ikbd_options_t		ikbd;
@@ -208,17 +256,20 @@ typedef struct {
   bx_cpu_options_t  cpu;
 #endif
   bx_autozoom_options_t	autozoom;
+  bx_nfosmesa_options_t	osmesa;
+  bx_parallel_options_t parallel;
+  bx_natfeat_options_t natfeats;
+  bx_nfvdi_options_t	nfvdi;
+#ifdef GDBSTUB
+  bx_gdbstub_t		gdbstub;
+#endif
   char			tos_path[512];
   char			emutos_path[512];
+  char			bootstrap_path[512];
+  char			bootdrive;
   uint32		fastram;
   bool			autoMouseGrab;
-  // char              bootdrive[2];
-  // unsigned long     vga_update_interval;
-  // unsigned long     keyboard_serial_delay;
-  // unsigned long     floppy_command_delay;
-  // unsigned long     ips;
-  // bool           private_colormap;
-  // bx_cmos_options_t	cmos;
+  bx_hotkeys_t		hotkeys;
   bool			newHardDriveSupport;
 } bx_options_t;
 
@@ -251,3 +302,7 @@ extern bool loadSettings(const char *);
 extern bool saveSettings(const char *);
 
 #endif
+
+/*
+vim:ts=4:sw=4:
+*/
