@@ -1,7 +1,7 @@
 /*
  * input.cpp - handling of keyboard/mouse input
  *
- * Copyright (c) 2001-2003 Petr Stehlik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2001-2005 Petr Stehlik of ARAnyM dev team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -32,6 +32,9 @@
 #  include "sdlgui.h"
 #endif
 
+#include "nf_objs.h"
+#include "nfvdi.h"
+
 #define DEBUG 0
 #include "debug.h"
 
@@ -52,21 +55,15 @@
 #define UNDEFINED_OFFSET	-1
 
 #ifdef OS_darwin
-#define KEYBOARD_TRANSLATION KEYSYM_MACSCANCODE
-#define HOTKEY_OPENGUI		SDLK_PRINT	// F13
-#define	HOTKEY_REBOOT		0
-#define	HOTKEY_SHUTDOWN		0
-#define	HOTKEY_FULLSCREEN	SDLK_NUMLOCK
-#define	HOTKEY_PRINT		SDLK_F15
+#define KEYBOARD_TRANSLATION	KEYSYM_MACSCANCODE
 #else
 #define KEYBOARD_TRANSLATION	KEYSYM_SCANCODE
-#define HOTKEY_OPENGUI		SDLK_PAUSE
-#define	HOTKEY_REBOOT		0
-#define	HOTKEY_SHUTDOWN		0
-#define	HOTKEY_FULLSCREEN	SDLK_SCROLLOCK
-#define	HOTKEY_PRINT		SDLK_PRINT
 #endif
 
+
+// according to Gerhard Stoll Milan defined scancode 76 for AltGr key
+#define RALT_ATARI_SCANCODE		(bx_options.ikbd.altgr ? 76 : 0x38)
+
 /*********************************************************************
  * Mouse handling
  *********************************************************************/
@@ -75,6 +72,18 @@ static bool grabbedMouse = false;
 static bool hiddenMouse = false;
 static bool capslockState = false;
 
+static void hideMouse(bool hide)
+{
+	if (hide) {
+		SDL_ShowCursor(SDL_DISABLE);
+		hiddenMouse = true;
+	}
+	else if (!hide) {
+		SDL_ShowCursor(SDL_ENABLE);
+		hiddenMouse = false;
+	}
+}
+
 void InputInit()
 {
 	// warp mouse to center of Atari 320x200 screen and grab it
@@ -89,23 +98,12 @@ void InputInit()
 
 void InputReset()
 {
+	// FIXME: add? InputInit();
 	// FIXME: how??? capslockState (detect)
 	capslockState = (SDL_GetModState() & KMOD_CAPS) != 0;
 }
 
 
-void hideMouse(bool hide)
-{
-	if (hide) {
-		SDL_ShowCursor(SDL_DISABLE);
-		hiddenMouse = true;
-	}
-	else if (!hide) {
-		SDL_ShowCursor(SDL_ENABLE);
-		hiddenMouse = false;
-	}
-}
-
 bool grabMouse(bool grab)
 {
 	int current = SDL_WM_GrabInput(SDL_GRAB_QUERY);
@@ -178,20 +176,22 @@ static int keyboardTable[0x80] = {
 /*78-7f*/ -1, -1, -1, -1, -1, -1, -1, -1
 };
 
-int keysymToAtari(SDL_keysym keysym)
+static int keysymToAtari(SDL_keysym keysym)
 {
  
-// fprintf (stdout, "scancode: %x - sym: %x - char: %s\n", keysym.scancode, keysym.sym, SDL_GetKeyName (keysym.sym));
+//fprintf (stdout, "scancode: %x - sym: %x - char: %s\n", keysym.scancode, keysym.sym, SDL_GetKeyName (keysym.sym));
 
 	int sym = keysym.scancode;
 
 	switch (keysym.sym) {
-	  case SDLK_RCTRL:
-	  case SDLK_LCTRL:
+	  case SDLK_LMETA:
+	  case SDLK_RMETA:
 	    return 0x1D;
 		break;
-	  case SDLK_LALT:
+	  case SDLK_MODE: /* passthru */ /* Alt Gr key according to SDL docs */
 	  case SDLK_RALT:
+		return RALT_ATARI_SCANCODE;
+	  case SDLK_LALT:
 	    return 0x38;
 		break;
 	  case SDLK_LSHIFT:
@@ -200,8 +200,8 @@ int keysymToAtari(SDL_keysym keysym)
 	  case SDLK_RSHIFT:
 	    return 0x36;
 		break;
-	  case SDLK_LMETA:
-	  case SDLK_RMETA:
+	  case SDLK_RCTRL:
+	  case SDLK_LCTRL:
 	    return 0;
 	  default:
 	    break;
@@ -249,15 +249,15 @@ static int keyboardTable[0x80] = {
 /*78-7f*/ 0, 0, 0, 0, 0, 0, 0, 0
 };
 
-int keysymToAtari(SDL_keysym keysym)
+static int keysymToAtari(SDL_keysym keysym)
 {
  
 	int sym = keysym.sym;
-	// map right Control and Alternate keys to the left ones
+	if (sym == SDLK_RALT || sym == SDLK_MODE /* Alt Gr */)
+		return RALT_ATARI_SCANCODE;
+	// map right Control key to the left one
 	if (sym == SDLK_RCTRL)
 		sym = SDLK_LCTRL;
-	if (sym == SDLK_RALT)
-		sym = SDLK_LALT;
 	for (int i = 0; i < 0x73; i++) {
 		if (keyboardTable[i] == sym) {
 			return i;
@@ -270,7 +270,7 @@ int keysymToAtari(SDL_keysym keysym)
 
 #if KEYBOARD_TRANSLATION == KEYSYM_SCANCODE
 // Heuristic analysis to find out the obscure scancode offset
-int findScanCodeOffset(SDL_keysym keysym)
+static int findScanCodeOffset(SDL_keysym keysym)
 {
 	int scanPC = keysym.scancode;
 	int offset = UNDEFINED_OFFSET;
@@ -340,7 +340,7 @@ int findScanCodeOffset(SDL_keysym keysym)
 	return offset;
 }
 
-int keysymToAtari(SDL_keysym keysym)
+static int keysymToAtari(SDL_keysym keysym)
 {
 	static int offset = UNDEFINED_OFFSET;
 
@@ -375,10 +375,10 @@ int keysymToAtari(SDL_keysym keysym)
 		case SDLK_INSERT:	return 0x52;	/* Insert */
 		case SDLK_DELETE:	return 0x53;	/* Delete */
 
-		// Map Right Alt/Alt Gr/Control to the Atari keys
-		case SDLK_RCTRL:	return 0x1d;	/* Control */
-		case SDLK_MODE:
-		case SDLK_RALT:		return 0x38;	/* Alternate */
+		// keys not found on original Atari keyboard
+		case SDLK_RCTRL:	return 0x1d;	/* map right Control to Atari control */
+		case SDLK_MODE: /* passthru */ /* Alt Gr key according to SDL docs */
+		case SDLK_RALT:		return RALT_ATARI_SCANCODE;
 
 		default:
 		{
@@ -417,7 +417,8 @@ extern bool isGuiAvailable;	// from main.cpp
 static SDL_Thread *GUIthread = NULL;
 static const int GUI_RETURN_INFO = (SDL_USEREVENT+1);
 
-int open_gui(void * /*ptr*/)
+// running in a different thread
+static int open_gui(void * /*ptr*/)
 {
 	hostScreen.openGUI();
 	int status = GUImainDlg();
@@ -435,7 +436,11 @@ int open_gui(void * /*ptr*/)
 
 bool start_GUI_thread()
 {
-	if (isGuiAvailable && !hostScreen.isGUIopen()) {
+	if (isGuiAvailable && !hostScreen.isGUIopen()
+#ifdef ENABLE_OPENGL
+		&& !bx_options.opengl.enabled
+#endif
+	) {
 		GUIthread = SDL_CreateThread(open_gui, NULL);
 	}
 	return (GUIthread != NULL);
@@ -448,17 +453,41 @@ void kill_GUI_thread()
 		GUIthread = NULL;
 	}
 }
-#endif
+#endif /* SDL_GUI */
 
-void process_keyboard_event(SDL_Event &event)
+#define CHECK_HOTKEY(Hotkey) ((bx_options.hotkeys.Hotkey.sym == 0 || sym == bx_options.hotkeys.Hotkey.sym) && masked_mod == bx_options.hotkeys.Hotkey.mod)
+static void process_keyboard_event(SDL_Event &event)
 {
-	bool pressed = (event.type == SDL_KEYDOWN);
 	SDL_keysym keysym = event.key.keysym;
 	SDLKey sym = keysym.sym;
-	int state = keysym.mod;	// SDL_GetModState();
+	int state = SDL_GetModState(); // keysym.mod does not deliver single mod key presses for some reason
+
+#ifdef SDL_GUI
+	if (hostScreen.isGUIopen()) {
+		SDL_Event ev;
+		ev.type = SDL_USEREVENT;	// map key down/up event to user event
+		ev.user.code = event.type;
+		ev.user.data1 = (void *)(uintptr)sym;
+		ev.user.data2 = (void *)(uintptr)state;
+		SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(SDL_USEREVENT));
+		return;	// don't pass the key events to emulation
+	}
+#endif /* SDL_GUI */
+
+#if FLIGHT_RECORDER
+	static bool flight_was_active = false;
+	bool flight_should_be_active = (state & (KMOD_SHIFT)) == KMOD_SHIFT;
+	if (flight_was_active != flight_should_be_active) {
+		cpu_flight_recorder(flight_should_be_active);
+		flight_was_active = flight_should_be_active;
+		panicbug("Flight was %sactivated!", (flight_should_be_active ? "" : "DE"));
+	}
+#endif
+
+	bool pressed = (event.type == SDL_KEYDOWN);
 	bool shifted = state & KMOD_SHIFT;
-	bool controlled = state & KMOD_CTRL;
-	bool alternated = state & KMOD_ALT;
+	// bool controlled = state & KMOD_CTRL;
+	// bool alternated = state & KMOD_ALT;
 	bool capslocked = state & KMOD_CAPS;
 	bool send2Atari = true;
 
@@ -474,47 +503,45 @@ void process_keyboard_event(SDL_Event &event)
 
 	// process special hotkeys
 	if (pressed) {
-		if (sym == SDLK_ESCAPE) {
-			if (controlled && alternated) {
-				if ( bx_options.video.fullscreen )
-					hostScreen.toggleFullScreen();
-				releaseTheMouse();
-				canGrabMouseAgain = false;	// let it leave our window
-				send2Atari = false;
-				// release the Control and Alt keys
-				getIKBD()->SendKey(0x1d|0x80);	// Control released
-				getIKBD()->SendKey(0x38|0x80);	// Alternate released
-			}
+		int masked_mod = state & HOTKEYS_MOD_MASK;
+
+		if (CHECK_HOTKEY(quit)) {
+			pendingQuit = true;
+			send2Atari = false;
 		}
-		else if (sym == HOTKEY_OPENGUI) {
-			if (shifted) {
-				pendingQuit = true;
-				send2Atari = false;
-			}
-			else if (controlled) {
-				send2Atari = false;
-				RestartAll();	// force Cold Reboot
-			}
+		else if (CHECK_HOTKEY(reboot)) {
+			RestartAll();	// force Cold Reboot
+			send2Atari = false;
+		}
+#ifdef SDL_GUI
+		else if (CHECK_HOTKEY(setup)) {
+			start_GUI_thread();
+			send2Atari = false;
+		}
+#endif
 #ifdef DEBUGGER
-			else if (bx_options.startup.debugger && alternated) {
+		else if (CHECK_HOTKEY(debug)) {
+			if (bx_options.startup.debugger) {
 				releaseTheMouse();
 				canGrabMouseAgain = false;	// let it leave our window
 				// activate debugger
 				activate_debugger();
-				send2Atari = false;
 			}
-#endif
-#ifdef SDL_GUI
-			else {
-				start_GUI_thread();
-			}
-#endif
+			send2Atari = false;
 		}
-		else if (sym == HOTKEY_PRINT) {
+#endif
+		else if (CHECK_HOTKEY(ungrab)) {
+			if ( bx_options.video.fullscreen )
+				hostScreen.toggleFullScreen();
+			releaseTheMouse();
+			canGrabMouseAgain = false;	// let it leave our window
+			send2Atari = false;
+		}
+		else if (CHECK_HOTKEY(screenshot)) {
 			hostScreen.makeSnapshot();
 			send2Atari = false;
 		}
-		else if (sym == HOTKEY_FULLSCREEN) {
+		else if (CHECK_HOTKEY(fullscreen)) {
 			hostScreen.toggleFullScreen();
 			if (bx_options.video.fullscreen && !grabbedMouse)
 				grabTheMouse();
@@ -522,18 +549,6 @@ void process_keyboard_event(SDL_Event &event)
 		}
 	}
 
-#ifdef SDL_GUI
-	if (hostScreen.isGUIopen()) {
-		SDL_Event ev;
-		ev.type = SDL_USEREVENT;	// map key down/up event to user event
-		ev.user.code = event.type;
-		ev.user.data1 = (void *)(uintptr)sym;
-		ev.user.data2 = (void *)(uintptr)state;
-		SDL_PeepEvents(&ev, 1, SDL_ADDEVENT, SDL_EVENTMASK(SDL_USEREVENT));
-		return;	// don't pass the key events to emulation
-	}
-#endif
-
 	// map special keys to Atari range of scancodes
 	if (sym == SDLK_PAGEUP) {
 		if (pressed) {
@@ -562,10 +577,11 @@ void process_keyboard_event(SDL_Event &event)
 		send2Atari = false;
 	}
 
+
 	// send all pressed keys to IKBD
 	if (send2Atari) {
 		int scanAtari = keysymToAtari(keysym);
-		D(bug("Host scancode = %d ($%02x), Atari scancode = %d ($%02x), keycode = '%s' ($%02x)", keysym.scancode, keysym.scancode, scanAtari, scanAtari, SDL_GetKeyName(sym), sym));
+		D(bug("Host scancode = %d ($%02x), Atari scancode = %d ($%02x), keycode = '%s' ($%02x)", keysym.scancode, keysym.scancode, pressed ? scanAtari : scanAtari|0x80, pressed ? scanAtari : scanAtari|0x80, SDL_GetKeyName(sym), sym));
 		if (scanAtari > 0) {
 			if (!pressed)
 				scanAtari |= 0x80;
@@ -574,8 +590,22 @@ void process_keyboard_event(SDL_Event &event)
 	}
 }
 
-void process_mouse_event(SDL_Event event)
+static void process_mouse_event(SDL_Event event)
 {
+	static NF_Base* fvdi = NULL;
+	bool fvdi_events = false;
+	bool mouse_exit = false;
+
+	if (!fvdi) {
+		unsigned int i;
+		for(i = 0; i < nf_objs_cnt; i++) {
+			if (strcasecmp("fVDI", nf_objects[i]->name()) == 0) {
+			    fvdi = nf_objects[i];
+			    break;
+			}
+		}
+	}
+
 #ifdef SDL_GUI
 	if (hostScreen.isGUIopen()) {
 		int typ = event.type;
@@ -589,7 +619,7 @@ void process_mouse_event(SDL_Event event)
 		}
 		return;	// don't pass the mouse events to emulation
 	}
-#endif
+#endif /* SDL_GUI */
 
 	int xrel = 0;
 	int yrel = 0;
@@ -607,6 +637,9 @@ void process_mouse_event(SDL_Event event)
 				grabTheMouse();
 		}
 		else if (event.button.button == 4) {	/* mouse wheel Up */
+			if (fvdi != NULL && fvdi->dispatch(0xc00100ff) == 0)
+				return;
+
 			if (bx_options.ikbd.wheel_eiffel) {
 				getIKBD()->SendKey(0xF6);
 				getIKBD()->SendKey(0x05);
@@ -623,6 +656,9 @@ void process_mouse_event(SDL_Event event)
 			return;
 		}
 		else if (event.button.button == 5) {	/* mouse wheel Down */
+			if (fvdi != NULL && fvdi->dispatch(0xc0010001) == 0)
+				return;
+
 			if (bx_options.ikbd.wheel_eiffel) {
 				getIKBD()->SendKey(0xF6);
 				getIKBD()->SendKey(0x05);
@@ -645,11 +681,13 @@ void process_mouse_event(SDL_Event event)
 		else if (event.button.button == SDL_BUTTON_LEFT)
 			but &= ~2;
 		else if (event.button.button == 4) {	/* mouse wheel Up */
-			getIKBD()->SendKey(0xc8);	// release keyUp
+			if (!bx_options.ikbd.wheel_eiffel)
+				getIKBD()->SendKey(0xc8);	// release keyUp
 			return;
 		}
 		else if (event.button.button == 5) {	/* mouse wheel Down */
-			getIKBD()->SendKey(0xd0);	// release keyDown
+			if (!bx_options.ikbd.wheel_eiffel)
+				getIKBD()->SendKey(0xd0);	// release keyDown
 			return;
 		}
 	}
@@ -657,6 +695,27 @@ void process_mouse_event(SDL_Event event)
 		SDL_MouseMotionEvent eve = event.motion;
 		xrel = eve.xrel;
 		yrel = eve.yrel;
+
+		if (fvdi != NULL) {
+			if (fvdi->dispatch(0x80008000 | (eve.x << 16) | (eve.y)) == 0)
+				fvdi_events = true;
+		}
+		// Can't use the method below to get out of the window
+		// if the events are reported directly, since it only
+		// works with hidden mouse pointer.
+		// So, define top left corner as exit point.
+		mouse_exit = (eve.x == 0) && (eve.y == 0);
+	}
+
+	if (lastbut != but) {
+		if (fvdi != NULL && (fvdi->dispatch(0xb0000000 | but) == 0))
+			fvdi_events = true;
+	}
+
+	if (fvdi_events) {
+		if (!bx_options.video.fullscreen && mouse_exit)
+			mouseOut = true;
+		return;
 	}
 
 	// send the mouse data packet
@@ -681,7 +740,7 @@ void process_mouse_event(SDL_Event event)
 	}
 }
 
-void process_active_event(SDL_Event event)
+static void process_active_event(SDL_Event event)
 {
 	// if we have input focus
 	if (SDL_GetAppState() & SDL_APPINPUTFOCUS) {
@@ -724,22 +783,25 @@ void process_active_event(SDL_Event event)
 
 SDL_Joystick *sdl_joystick;
 
-void process_joystick_event(SDL_Event event)
+static void process_joystick_event(SDL_Event event)
 {
 	switch(event.type) {
 		case SDL_JOYAXISMOTION:
-			getIKBD()->SendJoystickAxis(1, event.jaxis.axis,event.jaxis.value);
+			getIKBD()->SendJoystickAxis(1, event.jaxis.axis, event.jaxis.value);
+			break;		
+		case SDL_JOYHATMOTION:
+			getIKBD()->SendJoystickHat(1, event.jhat.value);
 			break;		
 		case SDL_JOYBUTTONDOWN:
 		case SDL_JOYBUTTONUP:
-			/* Only button 0 */
-			if (event.jbutton.button==0) {
-				getIKBD()->SendJoystickButton(1, event.jbutton.state==SDL_PRESSED);
-			}
+			getIKBD()->SendJoystickButton(1, event.jbutton.state==SDL_PRESSED);
 			break;		
 	}
 }
 
+///////
+// main function for checking keyboard, mouse and joystick events
+// called from main.cpp every 20 ms
 void check_event()
 {
 	if (!bx_options.video.fullscreen && mouseOut) {
@@ -757,6 +819,7 @@ void check_event()
 					| SDL_EVENTMASK(SDL_MOUSEBUTTONUP)
 					| SDL_EVENTMASK(SDL_MOUSEMOTION)
 					| SDL_EVENTMASK(SDL_JOYAXISMOTION)
+					| SDL_EVENTMASK(SDL_JOYHATMOTION)
 					| SDL_EVENTMASK(SDL_JOYBUTTONDOWN)
 					| SDL_EVENTMASK(SDL_JOYBUTTONUP)
 					| SDL_EVENTMASK(SDL_ACTIVEEVENT)
@@ -777,7 +840,7 @@ void check_event()
 			process_mouse_event(event);
 		}
 		else if (type == SDL_JOYAXISMOTION || type == SDL_JOYBUTTONDOWN
-			|| type == SDL_JOYBUTTONUP) {
+			|| type == SDL_JOYBUTTONUP || type == SDL_JOYHATMOTION) {
 			process_joystick_event(event);
 		}
 		else if (type == SDL_ACTIVEEVENT) {
@@ -791,7 +854,7 @@ void check_event()
 			else if (status == STATUS_REBOOT)
 				RestartAll();
 		}
-#endif
+#endif /* SDL_GUI */
 		else if (type == SDL_QUIT) {
 			pendingQuit = true;
 		}
@@ -801,3 +864,7 @@ void check_event()
 		Quit680x0();	// forces CPU to quit the loop
 	}
 }
+
+/*
+vim:ts=4:sw=4:
+*/
