@@ -1,5 +1,5 @@
 /*
- * $Header: /var/repos/aranym/src/ndebug.cpp,v 1.35 2004/07/28 20:10:09 milan Exp $
+ * $Header: /var/repos/aranym/src/ndebug.cpp,v 1.39 2005/05/05 22:47:02 milan Exp $
  *
  * MJ 2001
  */
@@ -17,13 +17,8 @@
 #define DEBUG 1
 #include "debug.h"
 
-#ifdef HAVE_NEW_HEADERS
 # include <cstdarg>
 # include <cstdlib>
-#else
-# include <stdarg.h>
-# include <stdlib.h>
-#endif
 
 #ifndef HAVE_GNU_SOURCE
 
@@ -34,6 +29,8 @@ extern "C" int vasprintf(char **, const char *, va_list);
 
 #include "parameters.h"
 
+extern void QuitEmulator(void);
+
 #ifdef NEWDEBUG
 
 #ifndef HAVE_STRDUP
@@ -157,13 +154,14 @@ int dbprintf(char *s, ...)
 {
 #endif
 	va_list a;
-	va_start(a, s);
 #ifdef NEWDEBUG
 	if (bx_options.startup.debugger) {
 		int i;
 		if (dbbuffer[dbend] != NULL)
 			free(dbbuffer[dbend]);
+		va_start(a, s);
 		i = vasprintf(&dbbuffer[dbend++], s, a);
+		va_end(a);
 		if (dbend == dbsize) dbend = 0;
 		if (dbstart == dbend) dbstart++;
 		if (dbstart == dbsize) dbstart = 0;
@@ -172,16 +170,14 @@ int dbprintf(char *s, ...)
 	} else {
 #endif
 		int i;
-		char *c;
-		i = vasprintf(&c, s, a);
-		i += fprintf(stderr, c);
+		va_start(a, s);
+		i = vfprintf(stderr, s, a);
 		i += fprintf(stderr, "\n");
-		free(c);
+		va_end(a);
 		return i;
 #ifdef NEWDEBUG
 	}
 #endif
-	va_end(a);
 }
 
 #ifdef NEWDEBUG
@@ -193,25 +189,24 @@ int pdbprintf(char *s, ...)
 #endif
 	int i = 0;
 	va_list a;
-	va_start(a, s);
 #ifdef NEWDEBUG
 	if (bx_options.startup.debugger) {
 		if (dbbuffer[dbend] != NULL)
 			free(dbbuffer[dbend]);
+		va_start(a, s);
 		i += vasprintf(&dbbuffer[dbend++], s, a);
+		va_end(a);
 		if (dbend == dbsize) dbend = 0;
 		if (dbstart == dbend) dbstart++;
 		if (dbstart == dbsize) dbstart = 0;
 		reset_aktualrow();
 	}
 #endif
-	char *c;
-	i += vasprintf(&c, s, a);
-	i += fprintf(stderr, c);
+	va_start(a, s);
+	i = vfprintf(stderr, s, a);
 	i += fprintf(stderr, "\n");
-	free(c);
-	return i;
 	va_end(a);
+	return i;
 }
 
 
@@ -219,23 +214,18 @@ int pdbprintf(char *s, ...)
 void ndebug::warn_print(FILE * f)
 {
 	unsigned int ar;
-	char *s = (char *) malloc((rowlen + 1) * sizeof(char));
 	for (unsigned int i = 0; i < get_warnlen(); i++) {
 		ar = (aktualrow + i) % dbsize;
 		if (dbbuffer[ar] != NULL
 			&& (((dbstart <= dbend) && (ar >= dbstart) && (ar <= dbend))
 			|| (dbstart > dbend) && (i < dbsize))) {
-			snprintf(s, rowlen, dbbuffer[ar]);
-			fprintf(f, s);
-			for (unsigned int j = 0; j < (rowlen - strlen(s)); j++)
-				putc(' ', f);
+			fprintf(f, "%-*.*s", rowlen, rowlen, dbbuffer[ar]);
 		} else {
 			for (unsigned int j = 0; j < rowlen; j++)
 				putc(' ', f);
 		}
 		fprintf(f, "|\n");
 	}
-	free(s);
 }
 
 void ndebug::m68k_print(FILE * f)
@@ -430,7 +420,7 @@ void ndebug::errorintofile(FILE *, char **inl) {
 
 
 void ndebug::saveintofile(FILE *, char **inl) {
-	volatile uae_u32 src, len;
+	uae_u32 src, len;
 	char *name;
 	FILE *fp;
 
@@ -554,10 +544,10 @@ void ndebug::writeintomem(FILE *, char **c) {
 	}
 }
 
-void ndebug::backtrace(FILE *f, volatile unsigned int lines) {
-	volatile uaecptr st = m68k_areg(regs,7);
+void ndebug::backtrace(FILE *f, unsigned int lines) {
+	uaecptr st = m68k_areg(regs,7);
 	while (lines-- > 0) {
-		if (valid_address(st, 0, 0, sz_long))
+		if (valid_address(st, 0, sz_long))
 			fprintf (f, "%08lx: %08x\n", (unsigned long)st, ReadAtariInt32(st));
 		else
 			fprintf (f, " unknown address\n");
@@ -568,23 +558,24 @@ void ndebug::backtrace(FILE *f, volatile unsigned int lines) {
 void ndebug::log2phys(FILE *, uaecptr addr) {
 #ifdef FULLMMU
 	if (regs.mmu_enabled) {
-setjmpagain:
-		JMP_BUF excep_env_old;
-		memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-	        int prb = SETJMP(excep_env);
-        	if (prb == 2) {
-			bug("Bus error");
-	                goto setjmpagain;
-        	} if (prb == 3) {
-			bug("Access error");
-			goto setjmpagain;
-		} if (prb != 0) {
-			bug("Unknonw access error - %d", prb);
-			goto setjmpagain;
+		SAVE_EXCEPTION;
+		TRY(prb) {
+			bug("MMU enabled: %08lx -> %08lx", (unsigned long)addr,
+			    (unsigned long)mmu_translate(addr, FC_DATA, 0, sz_long, 0));
 		}
-		bug("MMU enabled: %08lx -> %08lx",	(unsigned long)addr,
-			(unsigned long)mmu_translate(addr, FC_DATA, 0, m68k_getpc(), sz_long, 0));
-		memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+		CATCH(prb) {
+			switch (prb) {
+			case 2:
+				bug("Bus error");
+				break;
+			case 3:
+				bug("Access error");
+				break;
+			default:
+				bug("Unknonw access error - %d", int(prb));
+			}
+		}
+		RESTORE_EXCEPTION;
 	} else {
 #endif /* FULLMMU */
 		bug("MMU disabled: %08lx", (unsigned long)addr);
@@ -1166,36 +1157,33 @@ void ndebug::nexit() {
 	tcsetattr(0, TCSAFLUSH, &savetty);
 }
 
-void ndebug::dumpmem(FILE *f, volatile uaecptr addr, uaecptr * nxmem, volatile unsigned int lns)
+void ndebug::dumpmem(FILE *f, VOLATILE uaecptr addr, uaecptr * nxmem, unsigned int lns)
 {
-	JMP_BUF excep_env_old;
-	memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-	uaecptr a;
+	SAVE_EXCEPTION;
+	VOLATILE uaecptr a;
 	broken_in = 0;
 	for (; lns-- && !broken_in;) {
-		volatile int i;
+		VOLATILE int i;
 		fprintf(f, "%08lx: \n", (unsigned long)addr);
 		for (i = 0; i < 16; i++) {
-setjmpagain:
-        		if (SETJMP(excep_env) != 0) {
+			TRY(prb) {
+				a = addr < 0xff000000 ? addr : addr & 0x00ffffff;
+				if ((addr & 0xfff00000) == 0x00f00000) {
+					fprintf(f, "%04x ", (unsigned int) HWget_w(a));
+				}
+				else {
+					fprintf(f, "%04x ", (unsigned int) ReadAtariInt16(a));
+				}
+			}
+			CATCH(prb) {
 				fprintf (f, "UA   ");
-				addr += 2;
-				if (++i == 16) break;
-		                goto setjmpagain;
 		        }
-			a = addr < 0xff000000 ? addr : addr & 0x00ffffff;
-			if ((addr & 0xfff00000) == 0x00f00000) {
-				fprintf(f, "%04x ", (unsigned int) HWget_w(a));
-			}
-			else {
-				fprintf(f, "%04x ", (unsigned int) ReadAtariInt16(a));
-			}
 			addr += 2;
 		}
 		fprintf(f, "\n");
 	}
 	*nxmem = addr;
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+	RESTORE_EXCEPTION;
 }
 
 uae_u32 ndebug::readhex (char v, char **c)
@@ -1324,18 +1312,30 @@ void ndebug::convertNo(char **s) {
 #ifdef FULL_HISTORY
 void ndebug::showHistory(unsigned int count) {
 	unsigned int temp = lasthist;
+#ifdef NEED_TO_DEBUG_BADLY
+	struct regstruct save_regs = regs;
+	struct flag_struct save_flags = regflags;
+#endif
 	
 	while (count-- > 0 && temp != firsthist) {
 	    if (temp == 0) temp = MAX_HIST-1; else temp--;
 	}
 	bug("History:");
 	while (temp != lasthist) {
-#ifndef NEED_TO_DEBUG_BADLY
+#ifdef NEED_TO_DEBUG_BADLY
+		regs = history[temp];
+		regflags = historyf[temp];
+		m68k_dumpstate (NULL);
+#else
 		showDisasm(history[temp]);
 #endif
 		//bug("%04x", history[temp]);
 		if (++temp == MAX_HIST) temp = 0;
 	}
+#ifdef NEED_TO_DEBUG_BADLY
+	regs = save_regs;
+	regflags = save_flags;
+#endif
 }
 #endif
 
@@ -1343,6 +1343,18 @@ void ndebug::showHistory(unsigned int count) {
 
 /*
  * $Log: ndebug.cpp,v $
+ * Revision 1.39  2005/05/05 22:47:02  milan
+ * support of old headers removed
+ *
+ * Revision 1.38  2005/04/24 11:59:29  schwab
+ * - ndebug.cpp: fix stdarg usage, handle NEED_TO_DEBUG_BADLY in showHistory.
+ *
+ * Revision 1.37  2005/04/24 01:00:59  schwab
+ * Implement use of C++ exceptions instead of setjmp/longjmp for exception handling
+ *
+ * Revision 1.36  2005/01/22 16:02:33  joy
+ * extern decl of QuitEmulator
+ *
  * Revision 1.35  2004/07/28 20:10:09  milan
  * switch 'I' modified, debugger can ignore interupt mode
  *
