@@ -1,7 +1,7 @@
 /*
  * parameters.cpp - parameter init/load/save code
  *
- * Copyright (c) 2001-2004 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2005 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  MJ		Milan Jurik
@@ -30,15 +30,13 @@
 #include "config.h"
 #include "parameters.h"
 #include "tools.h"		// for safe_strncpy()
+#include "gdbstub.h"
+#include "host.h"
 
 #define DEBUG 0
 #include "debug.h"
 
-#ifdef HAVE_NEW_HEADERS
 # include <cstdlib>
-#else
-# include <stdlib.h>
-#endif
 
 #ifndef USE_JIT
 # define USE_JIT 0
@@ -64,6 +62,22 @@
 # define HOSTFS_SUPPORT 0
 #endif
 
+#ifndef NFCDROM_SUPPORT
+# define NFCDROM_SUPPORT 0
+#endif
+
+#ifndef NFCDROM_LINUX_SUPPORT
+# define NFCDROM_LINUX_SUPPORT 0
+#endif
+
+#ifndef NFOSMESA_SUPPORT
+# define NFOSMESA_SUPPORT 0
+#endif
+
+#ifndef NFJPEG_SUPPORT
+# define NFJPEG_SUPPORT 0
+#endif
+
 static struct option const long_options[] =
 {
 #ifndef FixedSizeFastRAM
@@ -71,8 +85,12 @@ static struct option const long_options[] =
 #endif
   {"floppy", required_argument, 0, 'a'},
   {"resolution", required_argument, 0, 'r'},
-#ifdef DEBUGGER
+#if (defined(DEBUGGER) || defined(GDBSTUB))
   {"debug", no_argument, 0, 'D'},
+# ifdef GDBSTUB
+  {"port", required_argument, 0, 'p'},
+  {"base", required_argument, 0, 'b'},
+# endif
 #endif
   {"fullscreen", no_argument, 0, 'f'},
   {"nomouse", no_argument, 0, 'N'},
@@ -85,7 +103,9 @@ static struct option const long_options[] =
   {"version", no_argument, 0, 'V'},
   {"config", required_argument, 0, 'c'},
   {"save", no_argument, 0, 's'},
+#ifdef SDL_GUI
   {"gui", no_argument, 0, 'G'},
+#endif
   {"swap-ide", no_argument, 0, 'S'},
   {"emutos", no_argument, 0, 'e'},
 #ifdef ENABLE_LILO
@@ -96,10 +116,13 @@ static struct option const long_options[] =
 
 #define TOS_FILENAME		"ROM"
 #define EMUTOS_FILENAME		"etos512k.img"
+#define FREEMINT_FILENAME	"mintara.prg"
 
 char *program_name;		// set by main()
 
+#ifdef SDL_GUI
 bool startupGUI = false;
+#endif
 
 bool boot_emutos = false;
 bool boot_lilo = false;
@@ -109,7 +132,7 @@ uint32 FastRAMSize;
 static char config_file[512];
 
 #if !defined(XIF_HOST_IP) && !defined(XIF_ATARI_IP) && !defined(XIF_NETMASK)
-# define XIF_TYPE	"ppp"
+# define XIF_TYPE	"ptp"
 # define XIF_TUNNEL	"tap0"
 # define XIF_HOST_IP	"192.168.0.1"
 # define XIF_ATARI_IP	"192.168.0.2"
@@ -131,6 +154,8 @@ struct Config_Tag global_conf[]={
 	{ "Floppy", Path_Tag, bx_options.floppy.path, sizeof(bx_options.floppy.path), 0},
 	{ "TOS", Path_Tag, bx_options.tos_path, sizeof(bx_options.tos_path), 0},
 	{ "EmuTOS", Path_Tag, bx_options.emutos_path, sizeof(bx_options.emutos_path), 0},
+	{ "Bootstrap", Path_Tag, bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path), 0},
+	{ "BootDrive", Char_Tag, &bx_options.bootdrive, 0, 0},
 	{ "AutoGrabMouse", Bool_Tag, &bx_options.autoMouseGrab, 0, 0},
 #ifdef ENABLE_EPSLIMITER
 	{ "EpsEnabled", Bool_Tag, &bx_options.cpu.eps_enabled, 0, 0},
@@ -139,7 +164,8 @@ struct Config_Tag global_conf[]={
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_global() {
+void preset_global()
+{
   bx_options.autoMouseGrab = true;
   strcpy(bx_options.floppy.path, "");
 #ifdef FixedSizeFastRAM
@@ -153,13 +179,15 @@ void preset_global() {
 #endif
 }
 
-void postload_global() {
+void postload_global()
+{
 #ifndef FixedSizeFastRAM
 	FastRAMSize = bx_options.fastram * 1024 * 1024;
 #endif
 }
 
-void presave_global() {
+void presave_global()
+{
 	bx_options.fastram = FastRAMSize / 1024 / 1024;
 }
 
@@ -172,63 +200,72 @@ struct Config_Tag startup_conf[]={
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_startup() {
+void preset_startup()
+{
   bx_options.startup.debugger = false;
   bx_options.startup.grabMouseAllowed = true;
+#ifdef GDBSTUB
+  bx_options.gdbstub.text_base = GDBSTUB_TEXT_BASE;
+  bx_options.gdbstub.data_base = GDBSTUB_DATA_BASE;
+  bx_options.gdbstub.bss_base = GDBSTUB_BSS_BASE;
+#endif
 }
 
-void postload_startup() {
+void postload_startup()
+{
 }
 
-void presave_startup() {
+void presave_startup()
+{
 }
 
 /*************************************************************************/
 struct Config_Tag jit_conf[]={
 	{ "JIT", Bool_Tag, &bx_options.jit.jit, 0, 0},
 	{ "JITFPU", Bool_Tag, &bx_options.jit.jitfpu, 0, 0},
-	{ "TuneAlignment", Bool_Tag, &bx_options.jit.tunealign, 0, 0},
-	{ "TuneNOPfill", Bool_Tag, &bx_options.jit.tunenop, 0, 0},
 	{ "JITCacheSize", Int_Tag, &bx_options.jit.jitcachesize, 0, 0},
 	{ "JITLazyFlush", Int_Tag, &bx_options.jit.jitlazyflush, 0, 0},
-        { "JITBlackList", String_Tag, &bx_options.jit.jitblacklist, sizeof(bx_options.jit.jitblacklist), 0},
+	{ "JITBlackList", String_Tag, &bx_options.jit.jitblacklist, sizeof(bx_options.jit.jitblacklist), 0},
+	{ "JITInline", Bool_Tag, &bx_options.jit.jitinline, 0, 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_jit() {
+void preset_jit()
+{
 	bx_options.jit.jit = false;
 	bx_options.jit.jitfpu = true;
 	bx_options.jit.jitcachesize = 8192;
 	bx_options.jit.jitlazyflush = 1;
-	bx_options.jit.tunealign = true;
-	bx_options.jit.tunenop = true;
 	strcpy(bx_options.jit.jitblacklist, "");
 }
 
-void postload_jit() {
+void postload_jit()
+{
 }
 
-void presave_jit() {
+void presave_jit()
+{
 }
 
 /*************************************************************************/
 struct Config_Tag tos_conf[]={
 	{ "Cookie_MCH", HexLong_Tag, &bx_options.tos.cookie_mch, 0, 0},
 	{ "RedirConsole", Bool_Tag, &bx_options.tos.redirect_CON, 0, 0},
-	{ "RedirPrinter", Bool_Tag, &bx_options.tos.redirect_PRT, 0, 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_tos() {
+void preset_tos()
+{
   bx_options.tos.redirect_CON = false;
-  bx_options.tos.redirect_PRT = false;
   bx_options.tos.cookie_mch = 0x00050000; // ARAnyM
 }
 
-void postload_tos() {
+void postload_tos()
+{
 }
 
-void presave_tos() {
+void presave_tos()
+{
 }
 
 /*************************************************************************/
@@ -240,43 +277,43 @@ struct Config_Tag video_conf[]={
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_video() {
+void preset_video()
+{
   bx_options.video.fullscreen = false;		// Boot in Fullscreen
   bx_options.video.boot_color_depth = -1;	// Boot in color depth
   bx_options.video.monitor = -1;			// preserve default NVRAM monitor
   bx_options.video.refresh = 2;			// 25 Hz update
 }
 
-void postload_video() {
+void postload_video()
+{
 	if (bx_options.video.refresh < 1 || bx_options.video.refresh > 200)
 		bx_options.video.refresh = 2;	// default if input parameter is insane
 }
 
-void presave_video() {
+void presave_video()
+{
 }
 
 /*************************************************************************/
 struct Config_Tag opengl_conf[]={
 	{ "Enabled", Bool_Tag, &bx_options.opengl.enabled, 0, 0},
-	{ "Width", Int_Tag, &bx_options.opengl.width, 0, 0},
-	{ "Height", Int_Tag, &bx_options.opengl.height, 0, 0},
-	{ "Bpp", Int_Tag, &bx_options.opengl.bpp, 0, 0},
 	{ "Filtered", Bool_Tag, &bx_options.opengl.filtered, 0, 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_opengl() {
+void preset_opengl()
+{
   bx_options.opengl.enabled = false;
-  bx_options.opengl.width = 640;
-  bx_options.opengl.height = 480;
-  bx_options.opengl.bpp = 16;
   bx_options.opengl.filtered = false;
 }
 
-void postload_opengl() {
+void postload_opengl()
+{
 }
 
-void presave_opengl() {
+void presave_opengl()
+{
 }
 
 /*************************************************************************/
@@ -296,31 +333,36 @@ void presave_opengl() {
 BX_DISK_CONFIG(diskc);
 BX_DISK_CONFIG(diskd);
 
-void set_ide(unsigned int number, char *dev_path, int cylinders, int heads, int spt, int byteswap, bool readonly, const char *model_name) {
+static void set_ide(unsigned int number, char *dev_path, int cylinders, int heads, int spt, int byteswap, bool readonly, const char *model_name)
+{
   // Autodetect ???
-  if (cylinders == -1)
+  if (cylinders == -1) {
     if ((cylinders = get_geometry(dev_path, geoCylinders)) == -1) {
       fprintf(stderr, "Disk %s has unknown geometry.\n", dev_path);
       exit(-1);
     }
+  }
 
-  if (heads == -1)
+  if (heads == -1) {
     if ((heads = get_geometry(dev_path, geoHeads)) == -1) {
       fprintf(stderr, "Disk %s has unknown geometry.\n", dev_path);
       exit(-1);
     }
+  }
 
-  if (spt == -1)
+  if (spt == -1) {
     if ((spt = get_geometry(dev_path, geoSpt)) == -1) {
       fprintf(stderr, "Disk %s has unknown geometry.\n", dev_path);
       exit(-1);
     }
+  }
 
-  if (byteswap == -1)
+  if (byteswap == -1) {
     if ((byteswap = get_geometry(dev_path, geoByteswap)) == -1) {
       fprintf(stderr, "Disk %s has unknown geometry.\n", dev_path);
       exit(-1);
     }
+  }
 
 	bx_atadevice_options_t *disk;
 	switch(number) {
@@ -346,29 +388,41 @@ void set_ide(unsigned int number, char *dev_path, int cylinders, int heads, int
 
 }
 
-void preset_ide() {
+void preset_ide()
+{
   set_ide(0, "", 0, 0, 0, false, false, "Master");
   set_ide(1, "", 0, 0, 0, false, false, "Slave");
 
   bx_options.newHardDriveSupport = true;
 }
 
-void postload_ide() {
+void postload_ide()
+{
 }
 
-void presave_ide() {
+void presave_ide()
+{
 }
 
 /*************************************************************************/
 #define DISK_CONFIG(Disk)	struct Config_Tag Disk ## _configs[] = {	\
-	{ "Path", Path_Tag, bx_options.Disk.path, sizeof(bx_options.Disk.path), 0},	\
-	{ "Present", Bool_Tag, &bx_options.Disk.present, 0, 0},	\
-	{ "PartID", String_Tag, bx_options.Disk.partID, sizeof(bx_options.Disk.partID), 0},	\
-	{ "ByteSwap", Bool_Tag, &bx_options.Disk.byteswap, 0, 0},	\
-	{ "ReadOnly", Bool_Tag, &bx_options.Disk.readonly, 0, 0},	\
+	{ "Path", Path_Tag, Disk->path, sizeof(Disk->path), 0},	\
+	{ "Present", Bool_Tag, &Disk->present, 0, 0},	\
+	{ "PartID", String_Tag, Disk->partID, sizeof(Disk->partID), 0},	\
+	{ "ByteSwap", Bool_Tag, &Disk->byteswap, 0, 0},	\
+	{ "ReadOnly", Bool_Tag, &Disk->readonly, 0, 0},	\
 	{ NULL , Error_Tag, NULL, 0, 0 }	\
 }
 
+static bx_scsidevice_options_t *disk0 = &bx_options.disks[0];
+static bx_scsidevice_options_t *disk1 = &bx_options.disks[1];
+static bx_scsidevice_options_t *disk2 = &bx_options.disks[2];
+static bx_scsidevice_options_t *disk3 = &bx_options.disks[3];
+static bx_scsidevice_options_t *disk4 = &bx_options.disks[4];
+static bx_scsidevice_options_t *disk5 = &bx_options.disks[5];
+static bx_scsidevice_options_t *disk6 = &bx_options.disks[6];
+static bx_scsidevice_options_t *disk7 = &bx_options.disks[7];
+
 DISK_CONFIG(disk0);
 DISK_CONFIG(disk1);
 DISK_CONFIG(disk2);
@@ -378,47 +432,61 @@ DISK_CONFIG(disk5);
 DISK_CONFIG(disk6);
 DISK_CONFIG(disk7);
 
-void preset_disk() {
+void preset_disk()
+{
+	for(int i=0; i<DISKS; i++) {
+		*bx_options.disks[i].path = '\0';
+		bx_options.disks[i].present = false;
+		strcpy(bx_options.disks[i].partID, "BGM");
+		bx_options.disks[i].byteswap = false;
+		bx_options.disks[i].readonly = false;
+	}
 }
 
-void postload_disk() {
+void postload_disk()
+{
 }
 
-void presave_disk() {
+void presave_disk()
+{
 }
 
 /*************************************************************************/
+#define HOSTFS_ENTRY(c,n) \
+	{ c, Path_Tag, &bx_options.aranymfs[n].configPath, sizeof(bx_options.aranymfs[n].configPath), 0}
+
 struct Config_Tag arafs_conf[]={
-	{ "A", Path_Tag, bx_options.aranymfs[0].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "B", Path_Tag, bx_options.aranymfs[1].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "C", Path_Tag, bx_options.aranymfs[2].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "D", Path_Tag, bx_options.aranymfs[3].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "E", Path_Tag, bx_options.aranymfs[4].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "F", Path_Tag, bx_options.aranymfs[5].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "G", Path_Tag, bx_options.aranymfs[6].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "H", Path_Tag, bx_options.aranymfs[7].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "I", Path_Tag, bx_options.aranymfs[8].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "J", Path_Tag, bx_options.aranymfs[9].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "K", Path_Tag, bx_options.aranymfs[10].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "L", Path_Tag, bx_options.aranymfs[11].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "M", Path_Tag, bx_options.aranymfs[12].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "N", Path_Tag, bx_options.aranymfs[13].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "O", Path_Tag, bx_options.aranymfs[14].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "P", Path_Tag, bx_options.aranymfs[15].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "Q", Path_Tag, bx_options.aranymfs[16].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "R", Path_Tag, bx_options.aranymfs[17].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "S", Path_Tag, bx_options.aranymfs[18].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "T", Path_Tag, bx_options.aranymfs[19].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "U", Path_Tag, bx_options.aranymfs[20].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "V", Path_Tag, bx_options.aranymfs[21].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "W", Path_Tag, bx_options.aranymfs[22].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "X", Path_Tag, bx_options.aranymfs[23].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "Y", Path_Tag, bx_options.aranymfs[24].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
-	{ "Z", Path_Tag, bx_options.aranymfs[25].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	HOSTFS_ENTRY("A", 0),
+	HOSTFS_ENTRY("B", 1),
+	HOSTFS_ENTRY("C", 2),
+	HOSTFS_ENTRY("D", 3),
+	HOSTFS_ENTRY("E", 4),
+	HOSTFS_ENTRY("F", 5),
+	HOSTFS_ENTRY("G", 6),
+	HOSTFS_ENTRY("H", 7),
+	HOSTFS_ENTRY("I", 8),
+	HOSTFS_ENTRY("J", 9),
+	HOSTFS_ENTRY("K", 10),
+	HOSTFS_ENTRY("L", 11),
+	HOSTFS_ENTRY("M", 12),
+	HOSTFS_ENTRY("N", 13),
+	HOSTFS_ENTRY("O", 14),
+	HOSTFS_ENTRY("P", 15),
+	HOSTFS_ENTRY("Q", 16),
+	HOSTFS_ENTRY("R", 17),
+	HOSTFS_ENTRY("S", 18),
+	HOSTFS_ENTRY("T", 19),
+	HOSTFS_ENTRY("U", 20),
+	HOSTFS_ENTRY("V", 21),
+	HOSTFS_ENTRY("W", 22),
+	HOSTFS_ENTRY("X", 23),
+	HOSTFS_ENTRY("Y", 24),
+	HOSTFS_ENTRY("Z", 25),
 	{ NULL , Error_Tag, NULL, 0, 0}
 };
 
-void preset_arafs() {
+void preset_arafs()
+{
 	for(int i=0; i < 'Z'-'A'+1; i++) {
 		bx_options.aranymfs[i].rootPath[0] = '\0';
 		bx_options.aranymfs[i].configPath[0] = '\0';
@@ -426,7 +494,8 @@ void preset_arafs() {
 	}
 }
 
-void postload_arafs() {
+void postload_arafs()
+{
 	for(int i=0; i < 'Z'-'A'+1; i++) {
 		safe_strncpy(bx_options.aranymfs[i].rootPath, bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].rootPath));
 		int len = strlen(bx_options.aranymfs[i].configPath);
@@ -441,7 +510,8 @@ void postload_arafs() {
 	}
 }
 
-void presave_arafs() {
+void presave_arafs()
+{
 	for(int i=0; i < 'Z'-'A'+1; i++) {
 		safe_strncpy(bx_options.aranymfs[i].configPath, bx_options.aranymfs[i].rootPath, sizeof(bx_options.aranymfs[i].configPath));
 		if ( strlen(bx_options.aranymfs[i].rootPath) > 0 &&
@@ -453,28 +523,52 @@ void presave_arafs() {
 }
 
 /*************************************************************************/
-#define ETH(x) bx_options.ethernet.x
-struct Config_Tag ethernet_conf[]={
-	{ "Type", String_Tag, &ETH(type), sizeof(ETH(type)), 0},
-	{ "Tunnel", String_Tag, &ETH(tunnel), sizeof(ETH(tunnel)), 0},
-	{ "HostIP", String_Tag, &ETH(ip_host), sizeof(ETH(ip_host)), 0},
-	{ "AtariIP", String_Tag, &ETH(ip_atari), sizeof(ETH(ip_atari)), 0},
-	{ "Netmask", String_Tag, &ETH(netmask), sizeof(ETH(netmask)), 0},
-	{ NULL , Error_Tag, NULL, 0, 0 }
-};
-
-void preset_ethernet() {
-  safe_strncpy(ETH(type), XIF_TYPE, sizeof(ETH(type)));
-  safe_strncpy(ETH(tunnel), XIF_TUNNEL, sizeof(ETH(tunnel)));
-  safe_strncpy(ETH(ip_host), XIF_HOST_IP, sizeof(ETH(ip_host)));
-  safe_strncpy(ETH(ip_atari ), XIF_ATARI_IP, sizeof(ETH(ip_atari)));
-  safe_strncpy(ETH(netmask), XIF_NETMASK, sizeof(ETH(netmask)));
+// struct Config_Tag ethernet_conf[]={
+#define ETH_CONFIG(Eth)	struct Config_Tag Eth ## _conf[] = {	\
+	{ "Type", String_Tag, &Eth->type, sizeof(Eth->type), 0}, \
+	{ "Tunnel", String_Tag, &Eth->tunnel, sizeof(Eth->tunnel), 0}, \
+	{ "HostIP", String_Tag, &Eth->ip_host, sizeof(Eth->ip_host), 0}, \
+	{ "AtariIP", String_Tag, &Eth->ip_atari, sizeof(Eth->ip_atari), 0}, \
+	{ "Netmask", String_Tag, &Eth->netmask, sizeof(Eth->netmask), 0}, \
+	{ NULL , Error_Tag, NULL, 0, 0 } \
 }
 
-void postload_ethernet() {
+static bx_ethernet_options_t *eth0 = &bx_options.ethernet[0];
+static bx_ethernet_options_t *eth1 = &bx_options.ethernet[1];
+static bx_ethernet_options_t *eth2 = &bx_options.ethernet[2];
+static bx_ethernet_options_t *eth3 = &bx_options.ethernet[3];
+
+ETH_CONFIG(eth0);
+ETH_CONFIG(eth1);
+ETH_CONFIG(eth2);
+ETH_CONFIG(eth3);
+
+#define ETH(i, x) bx_options.ethernet[i].x
+void preset_ethernet()
+{
+	// ETH[0] with some default values
+	safe_strncpy(ETH(0, type), XIF_TYPE, sizeof(ETH(0, type)));
+	safe_strncpy(ETH(0, tunnel), XIF_TUNNEL, sizeof(ETH(0, tunnel)));
+	safe_strncpy(ETH(0, ip_host), XIF_HOST_IP, sizeof(ETH(0, ip_host)));
+	safe_strncpy(ETH(0, ip_atari), XIF_ATARI_IP, sizeof(ETH(0, ip_atari)));
+	safe_strncpy(ETH(0, netmask), XIF_NETMASK, sizeof(ETH(0, netmask)));
+
+	// ETH[1] - ETH[MAX_ETH] are empty by default
+	for(int i=1; i<MAX_ETH; i++) {
+		*ETH(i, type) = '\0';
+		*ETH(i, tunnel) = '\0';
+		*ETH(i, ip_host) = '\0';
+		*ETH(i, ip_atari) = '\0';
+		*ETH(i, netmask) = '\0';
+	}
 }
 
-void presave_ethernet() {
+void postload_ethernet()
+{
+}
+
+void presave_ethernet()
+{
 }
 
 /*************************************************************************/
@@ -487,16 +581,19 @@ struct Config_Tag lilo_conf[]={
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
-void preset_lilo() {
+void preset_lilo()
+{
   safe_strncpy(LILO(kernel), "linux.bin", sizeof(LILO(kernel)));
   safe_strncpy(LILO(args), "root=/dev/ram video=atafb:vga16", sizeof(LILO(args)));
   safe_strncpy(LILO(ramdisk), "root.bin", sizeof(LILO(ramdisk)));
 }
 
-void postload_lilo() {
+void postload_lilo()
+{
 }
 
-void presave_lilo() {
+void presave_lilo()
+{
 }
 
 /*************************************************************************/
@@ -589,6 +686,159 @@ void postload_autozoom() {
 void presave_autozoom() {
 }
 
+/*************************************************************************/
+#define OSMESA_CONF(x) bx_options.osmesa.x
+
+struct Config_Tag osmesa_conf[]={
+	{ "ChannelSize", Int_Tag, &OSMESA_CONF(channel_size), 0, 0},
+	{ "LibGL", String_Tag, &OSMESA_CONF(libgl), sizeof(OSMESA_CONF(libgl)), 0},
+	{ "LibOSMesa", String_Tag, &OSMESA_CONF(libosmesa), sizeof(OSMESA_CONF(libosmesa)), 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_osmesa() {
+	OSMESA_CONF(channel_size) = 0;
+	safe_strncpy(OSMESA_CONF(libgl), "libGL.so", sizeof(OSMESA_CONF(libgl)));
+	safe_strncpy(OSMESA_CONF(libosmesa), "libOSMesa.so", sizeof(OSMESA_CONF(libosmesa)));
+}
+
+void postload_osmesa() {
+}
+
+void presave_osmesa() {
+}
+
+/*************************************************************************/
+#define PARALLEL_CONF(x) bx_options.parallel.x
+
+struct Config_Tag parallel_conf[]={
+	{ "Type", String_Tag, &PARALLEL_CONF(type), sizeof(PARALLEL_CONF(type)), 0},
+	{ "File", String_Tag, &PARALLEL_CONF(file), sizeof(PARALLEL_CONF(file)), 0},
+	{ "Parport", String_Tag, &PARALLEL_CONF(parport), sizeof(PARALLEL_CONF(parport)), 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_parallel() {
+  safe_strncpy(PARALLEL_CONF(type), "file", sizeof(PARALLEL_CONF(type)));
+  safe_strncpy(PARALLEL_CONF(file), "stderr", sizeof(PARALLEL_CONF(type)));
+  safe_strncpy(PARALLEL_CONF(parport), "/dev/parport0", sizeof(PARALLEL_CONF(parport)));
+}
+
+void postload_parallel() {
+}
+
+void presave_parallel() {
+}
+
+/*************************************************************************/
+#define NATFEAT_CONF(x) bx_options.natfeats.x
+
+struct Config_Tag natfeat_conf[]={
+	{ "CDROM", String_Tag, &NATFEAT_CONF(cdrom_driver), sizeof(NATFEAT_CONF(cdrom_driver)), 0},
+	{ "Vdi", String_Tag, &NATFEAT_CONF(vdi_driver), sizeof(NATFEAT_CONF(vdi_driver)), 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_natfeat() {
+  safe_strncpy(NATFEAT_CONF(cdrom_driver), "sdl", sizeof(NATFEAT_CONF(cdrom_driver)));
+  safe_strncpy(NATFEAT_CONF(vdi_driver), "soft", sizeof(NATFEAT_CONF(vdi_driver)));
+}
+
+void postload_natfeat() {
+}
+
+void presave_natfeat() {
+}
+
+/*************************************************************************/
+#define NFVDI_CONF(x) bx_options.nfvdi.x
+
+struct Config_Tag nfvdi_conf[]={
+	{ "UseHostMouseCursor", Bool_Tag,  &NFVDI_CONF(use_host_mouse_cursor), 0, 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_nfvdi() {
+	NFVDI_CONF(use_host_mouse_cursor) = false;
+}
+
+void postload_nfvdi() {
+}
+
+void presave_nfvdi() {
+}
+
+/*************************************************************************/
+#ifdef OS_darwin
+#define HOTKEY_OPENGUI		SDLK_PRINT	// F13
+#define	HOTKEY_FULLSCREEN	SDLK_NUMLOCK
+#define	HOTKEY_SCREENSHOT	SDLK_F15
+#else
+#define HOTKEY_OPENGUI		SDLK_PAUSE
+#define	HOTKEY_FULLSCREEN	SDLK_SCROLLOCK
+#define	HOTKEY_SCREENSHOT	SDLK_PRINT
+#endif
+
+#define HOTKEYS_STRING_SIZE		20
+static char hotkeys[10][HOTKEYS_STRING_SIZE];
+struct Config_Tag hotkeys_conf[]={
+	{ "Setup", String_Tag, hotkeys[0], HOTKEYS_STRING_SIZE, 0},
+	{ "Quit", String_Tag, hotkeys[1], HOTKEYS_STRING_SIZE, 0},
+	{ "Reboot", String_Tag, hotkeys[2], HOTKEYS_STRING_SIZE, 0},
+	{ "Ungrab", String_Tag, hotkeys[3], HOTKEYS_STRING_SIZE, 0},
+	{ "Debug", String_Tag, hotkeys[4], HOTKEYS_STRING_SIZE, 0},
+	{ "Screenshot", String_Tag, hotkeys[5], HOTKEYS_STRING_SIZE, 0},
+	{ "Fullscreen", String_Tag, hotkeys[6], HOTKEYS_STRING_SIZE, 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+struct { char *string; SDL_keysym *keysym; } hotkeys_rel[]={
+	{ hotkeys[0], &bx_options.hotkeys.setup },
+	{ hotkeys[1], &bx_options.hotkeys.quit },
+	{ hotkeys[2], &bx_options.hotkeys.reboot },
+	{ hotkeys[3], &bx_options.hotkeys.ungrab },
+	{ hotkeys[4], &bx_options.hotkeys.debug },
+	{ hotkeys[5], &bx_options.hotkeys.screenshot },
+	{ hotkeys[6], &bx_options.hotkeys.fullscreen }
+};
+
+void preset_hotkeys()
+{
+	// default values
+	bx_options.hotkeys.setup.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.setup.mod = KMOD_NONE;
+	bx_options.hotkeys.quit.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.quit.mod = KMOD_LSHIFT;
+	bx_options.hotkeys.reboot.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.reboot.mod = KMOD_LCTRL;
+	bx_options.hotkeys.ungrab.sym = SDLK_ESCAPE;
+	bx_options.hotkeys.ungrab.mod = (SDLMod)(KMOD_LSHIFT | KMOD_LCTRL | KMOD_LALT);
+	bx_options.hotkeys.debug.sym = HOTKEY_OPENGUI;
+	bx_options.hotkeys.debug.mod = KMOD_LALT;
+	bx_options.hotkeys.screenshot.sym = HOTKEY_SCREENSHOT;
+	bx_options.hotkeys.screenshot.mod = KMOD_NONE;
+	bx_options.hotkeys.fullscreen.sym = HOTKEY_FULLSCREEN;
+	bx_options.hotkeys.fullscreen.mod = KMOD_NONE;
+}
+
+void postload_hotkeys() {
+	// convert from string to pair of ints
+	for(uint i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
+		int sym, mod;
+		if ( sscanf(hotkeys_rel[i].string, "%i:%i", &sym, &mod) != 2 ) continue;
+
+		hotkeys_rel[i].keysym->sym = SDLKey(sym);
+		hotkeys_rel[i].keysym->mod = SDLMod(mod);
+	}
+}
+
+void presave_hotkeys() {
+	// convert from pair of ints to string
+	for(uint i=0; i<sizeof(hotkeys_rel)/sizeof(hotkeys_rel[0]); i++) {
+		snprintf(hotkeys_rel[i].string, HOTKEYS_STRING_SIZE, "%d:%#x", hotkeys_rel[i].keysym->sym, hotkeys_rel[i].keysym->mod);
+	}
+}
+
 /*************************************************************************/
 void usage (int status) {
   printf ("Usage: %s [OPTIONS]\n", program_name);
@@ -603,11 +853,13 @@ Options:\n\
   -m, --monitor <X>          attached monitor: 0 = VGA, 1 = TV\n\
   -c, --config FILE          read different configuration file\n\
   -s, --save                 save configuration file\n\
-  -G, --gui                  open GUI at startup\n\
   -S, --swap-ide             swap IDE drives\n\
   -h, --help                 display this help and exit\n\
   -V, --version              output version information and exit\n\
 ");
+#ifdef SDL_GUI
+  printf("  -G, --gui                  open GUI at startup\n");
+#endif
 #if HOSTFS_SUPPORT
   printf("  -d, --disk CHAR:PATH[:]    HostFS mapping, e.g. d:/atari/d_drive\n");
 #endif
@@ -617,14 +869,19 @@ Options:\n\
 #ifndef FixedSizeFastRAM
   printf("  -F, --fastram SIZE         FastRAM size (in MB)\n");
 #endif
-#ifdef DEBUGGER
+#if (defined(DEBUGGER) || defined(GDBSTUB))
   printf("  -D, --debug                start debugger\n");
+# ifdef GDBSTUB
+  printf("  -p, --port NUMBER          port number for GDB (default: 1234)\n");
+  printf("  -b, --base TEXT DATA BSS   base pointers\n");
+#endif
 #endif
   exit (status);
 }
 
 void preset_cfg() {
   preset_global();
+  preset_hotkeys();
   preset_ide();
   preset_disk();
   preset_arafs();
@@ -638,10 +895,15 @@ void preset_cfg() {
   preset_midi();
   preset_nfcdroms();
   preset_autozoom();
+  preset_osmesa();
+  preset_parallel();
+  preset_natfeat();
+  preset_nfvdi();
 }
 
 void postload_cfg() {
   postload_global();
+  postload_hotkeys();
   postload_ide();
   postload_disk();
   postload_arafs();
@@ -655,10 +917,15 @@ void postload_cfg() {
   postload_midi();
   postload_nfcdroms();
   postload_autozoom();
+  postload_osmesa();
+  postload_parallel();
+  postload_natfeat();
+  postload_nfvdi();
 }
 
 void presave_cfg() {
   presave_global();
+  presave_hotkeys();
   presave_ide();
   presave_disk();
   presave_arafs();
@@ -672,6 +939,10 @@ void presave_cfg() {
   presave_midi();
   presave_nfcdroms();
   presave_autozoom();
+  presave_osmesa();
+  presave_parallel();
+  presave_natfeat();
+  presave_nfvdi();
 }
 
 void early_cmdline_check(int argc, char **argv) {
@@ -699,6 +970,10 @@ void early_cmdline_check(int argc, char **argv) {
 			infoprint("DSP disassembler : %s", (DSP_DISASM == 1) ? "enabled" : "disabled");
 			infoprint("OpenGL support   : %s", (ENABLE_OPENGL == 1) ? "enabled" : "disabled");
 			infoprint("HOSTFS support   : %s", (HOSTFS_SUPPORT == 1) ? "enabled" : "disabled");
+			infoprint("Native features:");
+			infoprint(" CD-ROM driver   : %s", (NFCDROM_SUPPORT == 1) ? "enabled" : "disabled");
+			infoprint(" OSMesa rendering: %s", (NFOSMESA_SUPPORT == 1) ? "enabled" : "disabled");
+			infoprint(" JPEG decoder    : %s", (NFJPEG_SUPPORT == 1) ? "enabled" : "disabled");
 
 			exit (0);
 		}
@@ -714,8 +989,12 @@ int process_cmdline(int argc, char **argv)
 #ifdef ENABLE_LILO
 							 "l"  /* boot lilo */
 #endif
-#ifdef DEBUGGER
+#if (defined(DEBUGGER) || defined(GDBSTUB))
 							 "D"  /* debugger */
+# ifdef GDBSTUB
+							 "p:" /* gdb port */
+							 "b:" /* base */
+# endif
 #endif
 #ifndef FixedSizeFastRAM
 							 "F:" /* FastRAM */
@@ -731,7 +1010,9 @@ int process_cmdline(int argc, char **argv)
 							 "s"  /* save config file */
 
 							 "c:" /* path to config file */
+#ifdef SDL_GUI
 							 "G"  /* GUI startup */
+#endif
 							 "S"  /* swap IDE drives */
 							 "h"  /* help */
 							 "V"  /* version */,
@@ -743,15 +1024,31 @@ int process_cmdline(int argc, char **argv)
 			case 'S':
 				/* processed in early_cmdline_check already */
 				break;
-
+#ifdef SDL_GUI
 			case 'G':
 				startupGUI = true;
 				break;
-
-#ifdef DEBUGGER
+#endif
+#if (defined(DEBUGGER) || defined(GDBSTUB))
 			case 'D':
 				bx_options.startup.debugger = true;
 				break;
+# ifdef GDBSTUB
+			case 'p':
+				port_number = atoi(optarg); 
+				break;
+			case 'b':
+				if ((optind + 2 ) > argc)
+				{
+					fprintf(stderr, "Not enough parameters for  --base option");
+					usage(EXIT_FAILURE);
+				} else {
+					bx_options.gdbstub.text_base = atoi(optarg);
+					bx_options.gdbstub.data_base = atoi(argv[optind]);
+					bx_options.gdbstub.bss_base = atoi(argv[optind]);
+				}
+				break;
+# endif
 #endif
 
 			case 'e':
@@ -782,7 +1079,7 @@ int process_cmdline(int argc, char **argv)
 	
 			case 'a':
 				if ((strlen(optarg)-1) > sizeof(bx_options.floppy.path))
-					fprintf(stderr, "Floppy image filename longer than %lu chars.\n", sizeof(bx_options.floppy.path));
+					fprintf(stderr, "Floppy image filename longer than %zu chars.\n", sizeof(bx_options.floppy.path));
 				safe_strncpy(bx_options.floppy.path, optarg, sizeof(bx_options.floppy.path));
 				break;
 
@@ -841,11 +1138,11 @@ void postload_PathTag( const char *filename, struct Config_Tag *ptr ) {
 
 	if ( path[0] == '~' ) {
 		// replace with the home folder path
-		prefix = getHomeFolder(home, sizeof(home));
+		prefix = Host::getHomeFolder(home, sizeof(home));
 		prefixLen = strlen( prefix );
 		path++;
 	} else if (path[0] == '*') {
-		prefix=getDataFolder(home, sizeof(home));
+		prefix= Host::getDataFolder(home, sizeof(home));
 		prefixLen = strlen( prefix );
 		path++;
 	} else if ( path[0] != '/' && path[0] != '\\' && path[1] != ':' ) {
@@ -860,14 +1157,9 @@ void postload_PathTag( const char *filename, struct Config_Tag *ptr ) {
 	}
 
 	if ( prefixLen > 0 ) {
-		if (prefix[prefixLen-1]!=DIRSEPARATOR[0]) // if there isn't the last slash then add it 
-		{
-			prefixLen++;
-		}
 		if ( (size_t)ptr->buf_size >= prefixLen + strlen(path) ) {
 			memmove( (char*)ptr->buf + prefixLen, path, strlen(path)+1 );	
 			memmove( (char*)ptr->buf, prefix, prefixLen );
-			((char*)ptr->buf)[prefixLen-1]=DIRSEPARATOR[0];
 		} else {
 			fprintf(stderr, "Error - config entry size is insufficient\n" );
 		}
@@ -897,7 +1189,7 @@ char *addFilename(char *buffer, const char *file, unsigned int bufsize)
 // build a complete path to an user-specific file
 char *getConfFilename(const char *file, char *buffer, unsigned int bufsize)
 {
-	getConfFolder(buffer, bufsize);
+	Host::getConfFolder(buffer, bufsize);
 
 	// Does the folder exist?
 	struct stat buf;
@@ -912,7 +1204,7 @@ char *getConfFilename(const char *file, char *buffer, unsigned int bufsize)
 // build a complete path to system wide data file
 char *getDataFilename(const char *file, char *buffer, unsigned int bufsize)
 {
-	getDataFolder(buffer, bufsize);
+	Host::getDataFolder(buffer, bufsize);
 	return addFilename(buffer, file, bufsize);
 }
 
@@ -939,12 +1231,16 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 	if (stat(rcfile, &buf) == -1) {
 		fprintf(f, "Config file '%s' not found.\nThe config file is created with default values. Edit it to suit your needs.\n", rcfile);
 		saveConfigFile = true;
+#ifdef SDL_GUI
+		startupGUI = true;
+#endif
 		return false;
 	}
 
 	fprintf(f, "Using config file: '%s'\n", rcfile);
 
 	process_config(f, rcfile, global_conf, "[GLOBAL]", true);
+	process_config(f, rcfile, hotkeys_conf, "[HOTKEYS]", true);
 	process_config(f, rcfile, startup_conf, "[STARTUP]", true);
 	process_config(f, rcfile, jit_conf, "[JIT]", true);
 	process_config(f, rcfile, video_conf, "[VIDEO]", true);
@@ -952,28 +1248,29 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 	process_config(f, rcfile, ide_swap ? diskd_configs : diskc_configs, "[IDE0]", true);
 	process_config(f, rcfile, ide_swap ? diskc_configs : diskd_configs, "[IDE1]", true);
 
-/*
-	process_config(f, rcfile, disk0_configs, "[DISK0]", true);
-	process_config(f, rcfile, disk1_configs, "[DISK1]", true);
-	process_config(f, rcfile, disk2_configs, "[DISK2]", true);
-	process_config(f, rcfile, disk3_configs, "[DISK3]", true);
-	process_config(f, rcfile, disk4_configs, "[DISK4]", true);
-	process_config(f, rcfile, disk5_configs, "[DISK5]", true);
-	process_config(f, rcfile, disk6_configs, "[DISK6]", true);
-	process_config(f, rcfile, disk7_configs, "[DISK7]", true);
-*/
-	if (process_config(f, rcfile, arafs_conf, "[HOSTFS]", true) == 0) {
-		// fallback to obsolete [ARANYMFS]
-		fprintf(stderr, "[HOSTFS] section in config file not found.\n"
-						"falling back to obsolete [ARANYMFS]\n");
-		process_config(f, rcfile, arafs_conf, "[ARANYMFS]", true);
-	}
+	process_config(f, rcfile, disk0_configs, "[PARTITION0]", true);
+	process_config(f, rcfile, disk1_configs, "[PARTITION1]", true);
+	process_config(f, rcfile, disk2_configs, "[PARTITION2]", true);
+	process_config(f, rcfile, disk3_configs, "[PARTITION3]", true);
+	process_config(f, rcfile, disk4_configs, "[PARTITION4]", true);
+	process_config(f, rcfile, disk5_configs, "[PARTITION5]", true);
+	process_config(f, rcfile, disk6_configs, "[PARTITION6]", true);
+	process_config(f, rcfile, disk7_configs, "[PARTITION7]", true);
+
+	process_config(f, rcfile, arafs_conf, "[HOSTFS]", true);
 	process_config(f, rcfile, opengl_conf, "[OPENGL]", true);
-	process_config(f, rcfile, ethernet_conf, "[ETH0]", true);
+	process_config(f, rcfile, eth0_conf, "[ETH0]", true);
+	process_config(f, rcfile, eth1_conf, "[ETH1]", true);
+	process_config(f, rcfile, eth2_conf, "[ETH2]", true);
+	process_config(f, rcfile, eth3_conf, "[ETH3]", true);
 	process_config(f, rcfile, lilo_conf, "[LILO]", true);
 	process_config(f, rcfile, midi_conf, "[MIDI]", true);
 	process_config(f, rcfile, nfcdroms_conf, "[CDROMS]", true);
 	process_config(f, rcfile, autozoom_conf, "[AUTOZOOM]", true);
+	process_config(f, rcfile, osmesa_conf, "[NFOSMESA]", true);
+	process_config(f, rcfile, parallel_conf, "[PARALLEL]", true);
+	process_config(f, rcfile, natfeat_conf, "[NATFEATS]", true);
+	process_config(f, rcfile, nfvdi_conf, "[NFVDI]", true);
 
 	return true;
 }
@@ -993,30 +1290,49 @@ bool saveSettings(const char *fs)
 		fprintf(stderr, "Error while writing the '%s' config file.\n", fs);
 		return false;
 	}
+	update_config(fs, hotkeys_conf, "[HOTKEYS]");
 	update_config(fs, startup_conf, "[STARTUP]");
 	update_config(fs, jit_conf, "[JIT]");
 	update_config(fs, video_conf, "[VIDEO]");
 	update_config(fs, tos_conf, "[TOS]");
 	update_config(fs, ide_swap ? diskd_configs : diskc_configs, "[IDE0]");
 	update_config(fs, ide_swap ? diskc_configs : diskd_configs, "[IDE1]");
-/*
-	update_config(fs, disk0_configs, "[DISK0]");
-	update_config(fs, disk1_configs, "[DISK1]");
-	update_config(fs, disk2_configs, "[DISK2]");
-	update_config(fs, disk3_configs, "[DISK3]");
-	update_config(fs, disk4_configs, "[DISK4]");
-	update_config(fs, disk5_configs, "[DISK5]");
-	update_config(fs, disk6_configs, "[DISK6]");
-	update_config(fs, disk7_configs, "[DISK7]");
-*/
-	/// update_config(fs, arafs_conf, "[ARANYMFS]");
+
+	update_config(fs, disk0_configs, "[PARTITION0]");
+	if (strlen((char *)disk1_configs->buf))
+		update_config(fs, disk1_configs, "[PARTITION1]");
+	if (strlen((char *)disk2_configs->buf))
+		update_config(fs, disk2_configs, "[PARTITION2]");
+	if (strlen((char *)disk3_configs->buf))
+		update_config(fs, disk3_configs, "[PARTITION3]");
+	if (strlen((char *)disk4_configs->buf))
+		update_config(fs, disk4_configs, "[PARTITION4]");
+	if (strlen((char *)disk5_configs->buf))
+		update_config(fs, disk5_configs, "[PARTITION5]");
+	if (strlen((char *)disk6_configs->buf))
+		update_config(fs, disk6_configs, "[PARTITION6]");
+	if (strlen((char *)disk7_configs->buf))
+		update_config(fs, disk7_configs, "[PARTITION7]");
+
 	update_config(fs, arafs_conf, "[HOSTFS]");
 	update_config(fs, opengl_conf, "[OPENGL]");
-	update_config(fs, ethernet_conf, "[ETH0]");
+
+	update_config(fs, eth0_conf, "[ETH0]");
+	if (strlen((char *)eth1_conf->buf))
+		update_config(fs, eth1_conf, "[ETH1]");
+	if (strlen((char *)eth2_conf->buf))
+		update_config(fs, eth2_conf, "[ETH2]");
+	if (strlen((char *)eth3_conf->buf))
+		update_config(fs, eth3_conf, "[ETH3]");
+
 	update_config(fs, lilo_conf, "[LILO]");
 	update_config(fs, midi_conf, "[MIDI]");
 	update_config(fs, nfcdroms_conf, "[CDROMS]");
 	update_config(fs, autozoom_conf, "[AUTOZOOM]");
+	update_config(fs, osmesa_conf, "[NFOSMESA]");
+	update_config(fs, parallel_conf, "[PARALLEL]");
+	update_config(fs, natfeat_conf, "[NATFEATS]");
+	update_config(fs, nfvdi_conf, "[NFVDI]");
 
 	return true;
 }
@@ -1042,6 +1358,7 @@ bool decode_switches(FILE *f, int argc, char **argv)
 	getConfFilename(ARANYMCONFIG, config_file, sizeof(config_file));
 	getDataFilename(TOS_FILENAME, bx_options.tos_path, sizeof(bx_options.tos_path));
 	getDataFilename(EMUTOS_FILENAME, bx_options.emutos_path, sizeof(bx_options.emutos_path));
+	getDataFilename(FREEMINT_FILENAME, bx_options.bootstrap_path, sizeof(bx_options.bootstrap_path));
 
 	early_cmdline_check(argc, argv);
 	preset_cfg();
@@ -1060,3 +1377,7 @@ bool decode_switches(FILE *f, int argc, char **argv)
 const char *getConfigFile() {
 	return config_file;
 }
+
+/*
+vim:ts=4:sw=4:
+*/
