@@ -92,8 +92,8 @@ SDL_cond *stop_condition;
 bool InitMEM() {
 #if REAL_ADDRESSING
 	// Atari address space = host address space
-	RAMBase = (uint32)RAMBaseHost;
-	ROMBase = (uint32)ROMBaseHost;
+	RAMBase = (uintptr)RAMBaseHost;
+	ROMBase = (uintptr)ROMBaseHost;
 #else
 	InitMEMBaseDiff(RAMBaseHost, RAMBase);
 	InitROMBaseDiff(ROMBaseHost, ROMBase);
@@ -173,15 +173,6 @@ void Start680x0(void)
 	m68k_reset();
 #ifdef USE_JIT
 	if (bx_options.jit.jit) {
-# ifdef X86_ASSEMBLY_disable
-setjmpagain:
-	    int prb = SETJMP(excep_env);
-	    if (prb != 0) {
-		flush_icache(0);
-	        Exception(prb, 0);
-    		goto setjmpagain;
-	    }	
-# endif
 		m68k_compile_execute();
 	}
 	else
@@ -322,3 +313,8 @@ void report_double_bus_error()
 	panicbug(CPU_MSG);
 	CPU_ACTION;
 }
+
+#if FLIGHT_RECORDER
+extern bool cpu_flight_recorder_active;
+void cpu_flight_recorder(int activate) { cpu_flight_recorder_active = activate; }
+#endif
