@@ -41,13 +41,17 @@
 #include "input.h"
 #include "cpu_emulation.h"
 
+#include "gdbstub.h"
+
+#include "main.h"
+
 static int debugger_active = 0;
 int debugging = 0;
 int irqindebug = 0;
 
 int ignore_irq = 0;
 
-#ifndef NEWDEBUG
+#ifdef UAEDEBUG
 static int do_skip;
 static uaecptr skipaddr;
 static char old_debug_cmd[80];
@@ -59,8 +63,12 @@ void activate_debugger (void)
 {
 #ifdef NEWDEBUG
     ndebug::do_skip = false;
-#else
+#endif
+#ifdef UAEDEBUG
     do_skip = 0;
+#endif
+#ifdef GDBSTUB
+    gdbstub::init(port_number);
 #endif
     if (debugger_active)
 	return;
@@ -85,12 +93,13 @@ struct flag_struct historyf[MAX_HIST];
 uaecptr history[MAX_HIST];
 #endif
 
+#ifdef UAEDEBUG
+
 static void ignore_ws (char **c)
 {
     while (**c && isspace(**c)) (*c)++;
 }
 
-#ifndef NEWDEBUG
 static uae_u32 readhex (char **c)
 {
     uae_u32 val = 0;
@@ -110,7 +119,6 @@ static uae_u32 readhex (char **c)
     }
     return val;
 }
-#endif
 
 static char next_char( char **c)
 {
@@ -124,7 +132,6 @@ static int more_params (char **c)
     return (**c) != 0;
 }
 
-#ifndef NEWDEBUG
 static void dumpmem (uaecptr addr, uaecptr *nxmem, int lines)
 {
     broken_in = 0;
@@ -232,6 +239,10 @@ void debug (void)
 #ifdef NEWDEBUG
   ndebug::run();
 #else
+# ifdef GDBSTUB
+  if (gdbstub::check(m68k_getpc()) != GDBSTUB_STOP_NO_REASON)
+    gdbstub::debug_loop();
+# else
     char input[80];
     uaecptr nextpc,nxdis,nxmem;
 
@@ -306,7 +317,7 @@ void debug (void)
 	 case 'R': m68k_reset(); break;
 	 case 'A':
 	    {
-	        char reg;
+	        uae_u32 reg;
     	        if (more_params (&inptr))
 		    if ((reg = readhex(&inptr)) < 8)
 		        if (more_params (&inptr))
@@ -315,7 +326,7 @@ void debug (void)
 	    }
 	 case 'D':
 	    {
-	        char reg;
+	        uae_u32 reg;
     	        if (more_params (&inptr))
 		    if ((reg = readhex(&inptr)) < 8)
 		        if (more_params (&inptr))
@@ -347,7 +358,7 @@ void debug (void)
 		if (!more_params (&inptr))
 		    goto S_argh;
 		len = readhex (&inptr);
-		if (! valid_address (src, false, 0, len)) {
+		if (! valid_address (src, false, len)) {
 		    printf ("Invalid memory block\n");
 		    break;
 		}
@@ -479,5 +490,6 @@ void debug (void)
 
 	}
     }
-#endif
+# endif /* GDBSTUB */
+#endif /* NEWDEBUG */
 }
