@@ -97,14 +97,8 @@
  * - Precision rounding single/double
  */
 
+
 #include "sysdeps.h"
-#ifdef HAVE_NEW_HEADERS
-# include <cmath>
-# include <cstdio>
-#else
-# include <math.h>
-# include <stdio.h>
-#endif
 #include "memory.h"
 #include "readcpu.h"
 #include "newcpu.h"
@@ -113,6 +107,17 @@
 #include "fpu/fpu.h"
 #include "fpu/fpu_uae.h"
 
+#ifdef HAVE_NEW_HEADERS
+#define _GLIBCPP_USE_C99 1
+# include <cmath>
+# include <cstdio>
+using namespace __gnu_cxx;
+#undef _GLIBCPP_USE_C99
+#else
+# include <math.h>
+# include <stdio.h>
+#endif
+
 /* Global FPU context */
 fpu_t fpu;
 
