@@ -176,14 +176,12 @@ static const char *bit_mask (int size)
     return 0;
 }
 
-int nexti_no_inc = 0;
-
 static const char *gen_nextilong (void)
 {
     static char buffer[80];
     int r = m68k_pc_offset;
 
-    if (!nexti_no_inc) m68k_pc_offset += 4;
+    m68k_pc_offset += 4;
 
     if (using_prefetch)
 	sprintf (buffer, "get_ilong_prefetch(%d)", r);
@@ -197,7 +195,7 @@ static const char *gen_nextiword (void)
     static char buffer[80];
     int r = m68k_pc_offset;
 
-    if (!nexti_no_inc) m68k_pc_offset += 2;
+    m68k_pc_offset += 2;
 
     if (using_prefetch)
 	sprintf (buffer, "get_iword_prefetch(%d)", r);
@@ -233,7 +231,7 @@ static void fill_prefetch_2 (void)
 
 static void swap_opcode (void)
 {
-#ifdef HAVE_GET_WORD_UNSWAPPED
+#if ((defined(HAVE_GET_WORD_UNSWAPPED)) && (!defined(FULLMMU)))
   printf ("\topcode = do_byteswap_16(opcode);\n");
 #endif
 }
@@ -257,6 +255,13 @@ static void sync_m68k_pc (void)
     m68k_pc_offset = 0;
 }
 
+static void gen_set_fault_pc (void)
+{
+    sync_m68k_pc();
+    printf ("regs.fault_pc = m68k_getpc ();\n");
+    m68k_pc_offset = 0;
+}
+
 /* getv == 1: fetch data; getv != 0: check for odd address. If movem != 0,
  * the calling routine handles Apdi and Aipi modes.
  * gb-- movem == 2 means the same thing but for a MOVE16 instruction */
@@ -322,13 +327,19 @@ static void genamode (amodes mode, char *reg, wordsizes size, char *name, int ge
 	    if (movem)
 		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
 	    else
-		printf ("\tuaecptr %sa = m68k_areg(regs, %s) - areg_byteinc[%s];\n", name, reg, reg);
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= areg_byteinc[%s];\n", name, reg, reg);
 	    break;
 	 case sz_word:
-	    printf ("\tuaecptr %sa = m68k_areg(regs, %s) - %d;\n", name, reg, movem ? 0 : 2);
+	    if (movem)
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
+	    else
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= 2;\n", name, reg);
 	    break;
 	 case sz_long:
-	    printf ("\tuaecptr %sa = m68k_areg(regs, %s) - %d;\n", name, reg, movem ? 0 : 4);
+	    if (movem)
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s);\n", name, reg);
+	    else
+		printf ("\tuaecptr %sa = m68k_areg(regs, %s) -= 4;\n", name, reg);
 	    break;
 	 default:
 	    abort ();
@@ -463,9 +474,9 @@ static void genamode (amodes mode, char *reg, wordsizes size, char *name, int ge
 		abort ();
 	    }
 	    break;
-	 case Apdi:
+/*	 case Apdi:
 	    printf ("\tm68k_areg (regs, %s) = %sa;\n", reg, name);
-	    break;
+	    break;*/
 	 default:
 	    break;
 	}
@@ -511,8 +522,7 @@ static void genastore (char *from, amodes mode, char *reg, wordsizes size, char
      case absl:
      case PC16:
      case PC8r:
-	if (using_prefetch)
-	    sync_m68k_pc ();
+	gen_set_fault_pc ();
 	switch (size) {
 	 case sz_byte:
 	    printf ("\t%sput_byte(%sa,%s);\n", mem_prefix[xlateflag], to, from);
@@ -588,8 +598,7 @@ static void genmovemle (uae_u16 opcode)
     printf ("\tuae_u16 mask = %s;\n", gen_nextiword ());
     genamode (table68k[opcode].dmode, "dstreg", table68k[opcode].size, "src",
 			GENA_GETV_FETCH_ALIGN, GENA_MOVEM_NO_INC, XLATE_LOG);
-    if (using_prefetch)
-	sync_m68k_pc ();
+    sync_m68k_pc ();
 
     start_brace ();
     if (table68k[opcode].dmode == Apdi) {
@@ -771,7 +780,6 @@ static void genflags (flagtypes type, wordsizes size, char *value, char *src, ch
 	start_brace ();
 	printf ("\tuae_u32 %s;\n", value);
 	break;
-
      default:
 	break;
     }
@@ -832,7 +840,6 @@ static void genflags (flagtypes type, wordsizes size, char *value, char *src, ch
 	break;
     }
 #endif
-
     genflags_normal (type, size, value, src, dst);
 }
 
@@ -1206,9 +1213,8 @@ static void gen_opcode (unsigned long int opcode)
 	break;
      case i_TRAP:
 	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
-	sync_m68k_pc ();
+	gen_set_fault_pc ();
 	printf ("\tException(src+32,0);\n");
-	m68k_pc_offset = 0;
 	break;
      case i_MVR2USP:
 	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
@@ -1249,7 +1255,7 @@ static void gen_opcode (unsigned long int opcode)
 	    printf ("\telse if ((format & 0xF000) == 0x1000) { ; }\n");
 	    printf ("\telse if ((format & 0xF000) == 0x2000) { m68k_areg(regs, 7) += 4; break; }\n");
 //	    printf ("\telse if ((format & 0xF000) == 0x3000) { m68k_areg(regs, 7) += 4; break; }\n");
-	    printf ("\telse if ((format & 0xF000) == 0x7000) { m68k_areg(regs, 7) += 60; break; }\n");
+	    printf ("\telse if ((format & 0xF000) == 0x7000) { m68k_areg(regs, 7) += 52; break; }\n");
 	    printf ("\telse if ((format & 0xF000) == 0x8000) { m68k_areg(regs, 7) += 50; break; }\n");
 	    printf ("\telse if ((format & 0xF000) == 0x9000) { m68k_areg(regs, 7) += 12; break; }\n");
 	    printf ("\telse if ((format & 0xF000) == 0xa000) { m68k_areg(regs, 7) += 24; break; }\n");
@@ -1277,10 +1283,10 @@ static void gen_opcode (unsigned long int opcode)
      case i_LINK:
 	genamode (Apdi, "7", sz_long, "old", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_LOG);
 	genamode (curi->smode, "srcreg", sz_long, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
-	genastore ("src", Apdi, "7", sz_long, "old", xlateflag);
 	genastore ("m68k_areg(regs, 7)", curi->smode, "srcreg", sz_long, "src", xlateflag);
 	genamode (curi->dmode, "dstreg", curi->size, "offs", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
 	printf ("\tm68k_areg(regs, 7) += offs;\n");
+	genastore ("src", Apdi, "7", sz_long, "old", xlateflag);
 	break;
      case i_UNLK:
 	genamode (curi->smode, "srcreg", curi->size, "src", GENA_GETV_FETCH, GENA_MOVEM_DO_INC, XLATE_LOG);
@@ -1958,24 +1964,24 @@ static void gen_opcode (unsigned long int opcode)
 		start_brace();
 		printf ("\tif (extra & 0x0800)\n");	/* from reg to ea */
 		{
+			int old_m68k_pc_offset = m68k_pc_offset;
 			/* use DFC */
 			old_brace_level = n_braces;
 			start_brace ();
 			printf ("\tuae_u32 src = regs.regs[(extra >> 12) & 15];\n");
-			nexti_no_inc = 1; /* prevent strange problems with misaligned insns */
 #ifdef FULLMMU
 			genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_DFC);
 #else
 			genamode (curi->dmode, "dstreg", curi->size, "dst", GENA_GETV_FETCH_ALIGN, GENA_MOVEM_DO_INC, XLATE_LOG);
 						
 #endif
-			nexti_no_inc = 0;
 #ifdef FULLMMU
 			genastore ("src", curi->dmode, "dstreg", curi->size, "dst", XLATE_DFC);
 #else
 			genastore ("src", curi->dmode, "dstreg", curi->size, "dst", XLATE_LOG);
 #endif
 			pop_braces (old_brace_level);
+			m68k_pc_offset = old_m68k_pc_offset;
 		}
 		printf ("else");	/* from ea to reg */
 		{
@@ -1996,6 +2002,7 @@ static void gen_opcode (unsigned long int opcode)
 			printf ("\t} else {\n");
 			genastore ("src", Dreg, "(extra >> 12) & 7", curi->size, "", XLATE_LOG);
 			printf ("\t}\n");
+			sync_m68k_pc();
 			pop_braces (old_brace_level);
 		}
 	}
@@ -2121,11 +2128,11 @@ static void gen_opcode (unsigned long int opcode)
 	    printf ("\tm68k_dreg(regs, dstreg) = (m68k_dreg(regs, dstreg) & 0xffffff00) | ((val >> 4) & 0xf0) | (val & 0xf);\n");
 	} else {
 	    printf ("\tuae_u16 val;\n");
-	    printf ("\tm68k_areg(regs, srcreg) -= areg_byteinc[srcreg];\n");
-	    printf ("\tval = (uae_u16)get_byte(m68k_areg(regs, srcreg));\n");
-	    printf ("\tm68k_areg(regs, srcreg) -= areg_byteinc[srcreg];\n");
-	    printf ("\tval = (val | ((uae_u16)get_byte(m68k_areg(regs, srcreg)) << 8)) + %s;\n", gen_nextiword ());
+	    printf ("\tval = (uae_u16)get_byte(m68k_areg(regs, srcreg) - areg_byteinc[srcreg]);\n");
+	    printf ("\tval = (val | ((uae_u16)get_byte(m68k_areg(regs, srcreg) - 2 * areg_byteinc[srcreg]) << 8)) + %s;\n", gen_nextiword ());
+	    printf ("\tm68k_areg(regs, srcreg) -= 2;\n");
 	    printf ("\tm68k_areg(regs, dstreg) -= areg_byteinc[dstreg];\n");
+	    gen_set_fault_pc ();
 	    printf ("\tput_byte(m68k_areg(regs, dstreg),((val >> 4) & 0xf0) | (val & 0xf));\n");
 	}
 	break;
@@ -2136,13 +2143,12 @@ static void gen_opcode (unsigned long int opcode)
 	    printf ("\tm68k_dreg(regs, dstreg) = (m68k_dreg(regs, dstreg) & 0xffff0000) | (val & 0xffff);\n");
 	} else {
 	    printf ("\tuae_u16 val;\n");
-	    printf ("\tm68k_areg(regs, srcreg) -= areg_byteinc[srcreg];\n");
-	    printf ("\tval = (uae_u16)get_byte(m68k_areg(regs, srcreg));\n");
+	    printf ("\tval = (uae_u16)get_byte(m68k_areg(regs, srcreg) - areg_byteinc[srcreg]);\n");
 	    printf ("\tval = (((val << 4) & 0xf00) | (val & 0xf)) + %s;\n", gen_nextiword ());
-	    printf ("\tm68k_areg(regs, dstreg) -= areg_byteinc[dstreg];\n");
-	    printf ("\tput_byte(m68k_areg(regs, dstreg),val);\n");
-	    printf ("\tm68k_areg(regs, dstreg) -= areg_byteinc[dstreg];\n");
-	    printf ("\tput_byte(m68k_areg(regs, dstreg),val >> 8);\n");
+	    printf ("\tm68k_areg(regs, srcreg) -= areg_byteinc[srcreg];\n");
+	    printf ("\tm68k_areg(regs, dstreg) -= 2;\n");
+	    gen_set_fault_pc ();
+	    printf ("\tput_word(m68k_areg(regs, dstreg), val);\n");
 	}
 	break;
      case i_TAS:
@@ -2431,7 +2437,7 @@ static void generate_one_opcode (int rp)
 	    if (pos < 8 && (smsk >> (8 - pos)) != 0)
 		abort ();
 #endif
-#ifdef HAVE_GET_WORD_UNSWAPPED
+#if ((defined(HAVE_GET_WORD_UNSWAPPED)) && (!defined(FULLMMU)))
 
 	    if (pos < 8 && (smsk >> (8 - pos)) != 0)
 		sprintf (source, "(((opcode >> %d) | (opcode << %d)) & %d)",
@@ -2482,8 +2488,8 @@ static void generate_one_opcode (int rp)
 	    /* Check that we can do the little endian optimization safely.  */
 	    if (pos < 8 && (dmsk >> (8 - pos)) != 0)
 		abort ();
-#endif	    
-#ifdef HAVE_GET_WORD_UNSWAPPED
+#endif
+#if ((defined(HAVE_GET_WORD_UNSWAPPED)) && (!defined(FULLMMU)))
 
 	    if (pos < 8 && (dmsk >> (8 - pos)) != 0)
 		printf ("\tuae_u32 dstreg = ((opcode >> %d) | (opcode << %d)) & %d;\n",
@@ -2519,23 +2525,25 @@ static void generate_one_opcode (int rp)
     printf ("\t\t\tLONGJMP(loop_env, 1);\n");
     printf ("\t}\n");
     printf ("\tpc = m68k_getpc();\n");
+#ifndef FULLMMU
 #if ARAM_PAGE_CHECK
     printf ("\tif (((pc ^ pc_page) > ARAM_PAGE_MASK)) {\n");
     printf ("\t\tcheck_ram_boundary(pc, 2, false);\n");
     printf ("\t\tpc_page = pc;\n");
     printf ("\t\tpc_offset = (uae_u32)get_real_address(pc, 0, sz_word) - pc;\n");
     printf ("\t}\n");
-#else
+#else /* ARAM_PAGE_CHECK */
     printf ("\tcheck_ram_boundary(pc, 2, false);\n");
-#endif
+#endif /* ARAM_PAGE_CHECK */
+#endif /* !FULLMMU */
     printf ("\topcode = GET_OPCODE;}\n");
 #if 0
     printf ("\tprintf(\"%%lx 0x%%08x 0x%%08x\\n\", opcode, op_smalltbl_0_lab,op_smalltbl_0_lab[opcode]);\n");
 #endif
     printf ("\tgoto *op_smalltbl_0_lab[opcode];\n\n");
-#else
+#else /* DISDIP */
     printf ("\tcpuop_end();\n");
-#endif
+#endif /* DISDIP */
     printf ("}\n");
     opcode_next_clev[rp] = next_cpu_level;
     opcode_last_postfix[rp] = postfix;
@@ -2613,5 +2621,7 @@ int main ()
     generate_includes (stblfile);
     generate_func ();
     free (table68k);
+    fclose(headerfile);
+    fclose(stblfile);
     return 0;
 }
