@@ -1,7 +1,7 @@
 /*
  * memory.h - memory management
  *
- * Copyright (c) 2001-2004 Milan Jurik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2001-2006 Milan Jurik of ARAnyM dev team (see AUTHORS)
  * 
  * Inspired by Christian Bauer's Basilisk II
  *
@@ -41,15 +41,36 @@
 #include "cpummu.h"
 #include "readcpu.h"
 
-#ifdef HAVE_NEW_HEADERS
 # include <csetjmp>
-#else
-# include <setjmp.h>
-#endif
 
 // newcpu.h
 extern void Exception (int, uaecptr);
-extern JMP_BUF excep_env;
+#ifdef EXCEPTIONS_VIA_LONGJMP
+	extern JMP_BUF excep_env;
+	#define SAVE_EXCEPTION \
+		JMP_BUF excep_env_old; \
+		memcpy(excep_env_old, excep_env, sizeof(JMP_BUF))
+	#define RESTORE_EXCEPTION \
+		memcpy(excep_env, excep_env_old, sizeof(JMP_BUF))
+	#define TRY(var) int var = SETJMP(excep_env); if (!var)
+	#define CATCH(var) else
+	#define THROW(n) LONGJMP(excep_env, n)
+	#define THROW_AGAIN(var) LONGJMP(excep_env, var)
+	#define VOLATILE volatile
+#else
+	struct m68k_exception {
+		int prb;
+		m68k_exception (int exc) : prb (exc) {}
+		operator int() { return prb; }
+	};
+	#define SAVE_EXCEPTION
+	#define RESTORE_EXCEPTION
+	#define TRY(var) try
+	#define CATCH(var) catch(m68k_exception var)
+	#define THROW(n) throw m68k_exception(n)
+	#define THROW_AGAIN(var) throw
+	#define VOLATILE
+#endif /* EXCEPTIONS_VIA_LONGJMP */
 extern int in_exception_2;
 
 #define STRAM_END	0x0e00000UL	// should be replaced by global ROMBase as soon as ROMBase will be a constant
@@ -124,7 +145,8 @@ static inline void check_ram_boundary(uaecptr addr, int size, bool write)
 
 	// D(bug("BUS ERROR %s at $%x\n", (write ? "writing" : "reading"), addr));
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, 2);
+	regs.mmu_ssw = ((size & 3) << 5) | (write ? 0 : (1 << 8));
+	THROW(2);
 }
 
 #else
@@ -134,11 +156,7 @@ static inline void check_ram_boundary(uaecptr, int, bool) { }
 #ifdef FIXED_VIDEORAM
 # define do_get_real_address(a)		((uae_u8 *)(((uaecptr)(a) < ARANYMVRAMSTART) ? ((uaecptr)(a) + MEMBaseDiff) : ((uaecptr)(a) + VMEMBaseDiff)))
 #else
-# if defined (OS_darwin) && !defined (MACOSX_support) // not correct fixed position of allocations
-#  define do_get_real_address(a)         ((uae_u8 *)(((uintptr)(a) < STRAM_END) ? ((uaecptr)(a) + MEMBaseDiff) : ((((uintptr)(a) >= FastRAM_BEGIN) ? ((uaecptr)(a) + FastRAMBaseDiff) : ((uaecptr)(a) + ROMBaseDiff)))))
-# else
-#define do_get_real_address(a)		((uae_u8 *)((uintptr)(a) + MEMBaseDiff))
-# endif
+# define do_get_real_address(a)		((uae_u8 *)((uintptr)(a) + MEMBaseDiff))
 #endif
 
 static inline uae_u8 *phys_get_real_address(uaecptr addr)
@@ -149,15 +167,15 @@ static inline uae_u8 *phys_get_real_address(uaecptr addr)
 #ifndef NOCHECKBOUNDARY
 static inline bool phys_valid_address(uaecptr addr, bool write, int sz)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-        memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
-        return false;
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	check_ram_boundary(addr, sz, write);
     }
-    check_ram_boundary(addr, sz, write);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
+	return false;
+    }
+    RESTORE_EXCEPTION;
     return true;
 }
 #else
@@ -294,94 +312,114 @@ static inline void phys_put_byte(uaecptr addr, uae_u32 b)
 }
 
 #ifdef FULLMMU
+static inline bool is_unaligned(uaecptr addr, int size)
+{
+    uae_u32 page_mask = (regs.mmu_pagesize == MMU_PAGE_4KB ? 0xfffff000 : 0xffffe000);
+    return ((addr & (size - 1)) && (addr ^ (addr + size - 1)) & page_mask);
+}
+
 static inline uae_u32 get_long(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 4))
+	return mmu_get_unaligned(addr, FC_DATA, 4);
+    SAVE_EXCEPTION;
     uae_u32 l;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	l = phys_get_long(mmu_translate(addr, FC_DATA, 0, sz_long, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    l = phys_get_long(mmu_translate(addr, FC_DATA, 0, m68k_getpc(), sz_long, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return l;
 }
 
 static inline uae_u16 get_word(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 2))
+	return mmu_get_unaligned(addr, FC_DATA, 2);
+    SAVE_EXCEPTION;
     uae_u16 w;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	w = phys_get_word(mmu_translate(addr, FC_DATA, 0, sz_word, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    w = phys_get_word(mmu_translate(addr, FC_DATA, 0, m68k_getpc(), sz_word, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return w;
 }
 
 static inline uae_u8 get_byte(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    SAVE_EXCEPTION;
     uae_u8 b;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	b = phys_get_byte(mmu_translate(addr, FC_DATA, 0, sz_byte, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    b = phys_get_byte(mmu_translate(addr, FC_DATA, 0, m68k_getpc(), sz_byte, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return b;
 }
 
 static inline void put_long(uaecptr addr, uae_u32 l)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 4))
+	return mmu_put_unaligned(addr, l, FC_DATA, 4);
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_long(mmu_translate(addr, FC_DATA, 1, sz_long, 0),l);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = l;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_long(mmu_translate(addr, FC_DATA, 1, m68k_getpc(), sz_long, 0),l);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
 static inline void put_word(uaecptr addr, uae_u16 w)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 2))
+	return mmu_put_unaligned(addr, w, FC_DATA, 2);
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_word(mmu_translate(addr, FC_DATA, 1, sz_word, 0),w);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = w;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_word(mmu_translate(addr, FC_DATA, 1, m68k_getpc(), sz_word, 0),w);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
 static inline void put_byte(uaecptr addr, uae_u16 b)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_byte(mmu_translate(addr, FC_DATA, 1, sz_byte, 0),b);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = b;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_byte(mmu_translate(addr, FC_DATA, 1, m68k_getpc(), sz_byte, 0),b);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
 static inline uae_u8 *get_real_address(uaecptr addr, int write, int sz)
@@ -391,112 +429,126 @@ static inline uae_u8 *get_real_address(uaecptr addr, int write, int sz)
         case 1: i = sz_byte; break;
         case 2: i = sz_word; break;
     }
-    return phys_get_real_address(mmu_translate(addr, FC_DATA, write, m68k_getpc(), i, 0));
+    return phys_get_real_address(mmu_translate(addr, FC_DATA, write, i, 0));
 }
 
 static inline uae_u32 sfc_get_long(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 4))
+	return mmu_get_unaligned(addr, regs.sfc, 4);
+    SAVE_EXCEPTION;
     uae_u32 l;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	l = phys_get_long(mmu_translate(addr, regs.sfc, 0, sz_long, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    l = phys_get_long(mmu_translate(addr, regs.sfc, 0, m68k_getpc(), sz_long, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return l;
 }
 static inline uae_u16 sfc_get_word(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 1))
+	return mmu_get_unaligned(addr, regs.dfc, 2);
+    SAVE_EXCEPTION;
     uae_u16 w;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	w = phys_get_word(mmu_translate(addr, regs.sfc, 0, sz_word, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    w = phys_get_word(mmu_translate(addr, regs.sfc, 0, m68k_getpc(), sz_word, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return w;
 }
 static inline uae_u8 sfc_get_byte(uaecptr addr)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    SAVE_EXCEPTION;
     uae_u8 b;
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    TRY(prb) {
+	b = phys_get_byte(mmu_translate(addr, regs.sfc, 0, sz_byte, 0));
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	THROW_AGAIN(prb);
     }
-    b = phys_get_byte(mmu_translate(addr, regs.sfc, 0, m68k_getpc(), sz_byte, 0));
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
     return b;
 }
 
 static inline void dfc_put_long(uaecptr addr, uae_u32 l)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 4))
+	return mmu_put_unaligned(addr, l, regs.dfc, 4);
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_long(mmu_translate(addr, regs.dfc, 1, sz_long, 0), l);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = l;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_long(mmu_translate(addr, regs.dfc, 1, m68k_getpc(), sz_long, 0), l);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 static inline void dfc_put_word(uaecptr addr, uae_u16 w)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    if (is_unaligned(addr, 1))
+	return mmu_put_unaligned(addr, w, regs.dfc, 2);
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_word(mmu_translate(addr, regs.dfc, 1, sz_word, 0), w);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = w;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_word(mmu_translate(addr, regs.dfc, 1, m68k_getpc(), sz_word, 0), w);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 static inline void dfc_put_byte(uaecptr addr, uae_u16 b)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	phys_put_byte(mmu_translate(addr, regs.dfc, 1, sz_byte, 0), b);
+	RESTORE_EXCEPTION;
+    }
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	regs.mmu_fault_addr = addr;
-	LONGJMP(excep_env, prb);
+	regs.wb3_data = b;
+	regs.wb3_status = (regs.mmu_ssw & 0x7f) | 0x80;
+	THROW_AGAIN(prb);
     }
-    phys_put_byte(mmu_translate(addr, regs.dfc, 1, m68k_getpc(), sz_byte, 0), b);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
-static inline bool valid_address(uaecptr addr, bool write, uaecptr pc, int sz)
+static inline bool valid_address(uaecptr addr, bool write, int sz)
 {
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-    int prb = SETJMP(excep_env);
-    wordsizes i = sz_long;
-    switch (sz) {
-        case 1: i = sz_byte; break;
-        case 2: i = sz_word; break;
+    SAVE_EXCEPTION;
+    TRY(prb) {
+	wordsizes i = sz_long;
+	switch (sz) {
+	    case 1: i = sz_byte; break;
+	    case 2: i = sz_word; break;
+	}
+	check_ram_boundary(mmu_translate(addr, FC_DATA, (write ? 1 : 0), i, 0), sz, write);
+	RESTORE_EXCEPTION;
+	return true;
     }
-    if (prb != 0) {
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    CATCH(prb) {
+	RESTORE_EXCEPTION;
 	return false;
     } 
-    check_ram_boundary(mmu_translate(addr, FC_DATA, (write ? 1 : 0), pc, i, 0), sz, write);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
-    return true;
 }
 
 #else
@@ -509,7 +561,7 @@ static inline bool valid_address(uaecptr addr, bool write, uaecptr pc, int sz)
 #  define put_byte(a,b)			phys_put_byte(a,b)
 #  define get_real_address(a,w,s)	phys_get_real_address(a)
 
-#define valid_address(a,w,p,s)		phys_valid_address(a,w,s)
+#define valid_address(a,w,s)		phys_valid_address(a,w,s)
 #endif
 
 static inline void flush_internals() {
