@@ -43,6 +43,7 @@
 # include "compiler/compemu.h"
 #endif
 #include "fpu/fpu.h"
+#include "natfeats.h"
 
 #include <cstdlib>
 
@@ -59,7 +60,9 @@ bool cpu_debugging = false;
 struct flag_struct regflags;
 
 /* LongJump buffers */
+#ifdef EXCEPTIONS_VIA_LONGJMP
 JMP_BUF excep_env;
+#endif
 #ifdef DISDIP
 JMP_BUF loop_env;
 #endif
@@ -95,8 +98,11 @@ struct rec_step {
 	uae_u16 sr;
 	uae_u32 msp;
 	uae_u32 isp;
+	uae_u16 instr;
 };
 
+bool cpu_flight_recorder_active = false;
+
 const int LOG_SIZE = 8192;
 static rec_step log[LOG_SIZE];
 static int log_ptr = -1; // First time initialization
@@ -107,8 +113,29 @@ static const char *log_filename(void)
 	return name ? name : "log.68k";
 }
 
+static void dump_log();
 void m68k_record_step(uaecptr pc)
 {
+	static bool last_state = false;
+	if (! cpu_flight_recorder_active) {
+		if (last_state) {
+			// dump log out
+		    	dump_log();
+
+			// remember last state
+			last_state = false;
+		}
+		return;
+	}
+
+	if (! last_state) {
+		// reset old log
+    	log_ptr = 0;
+    	memset(log, 0, sizeof(log));
+		// remember last state
+		last_state = true;
+	}
+
 	for (int i = 0; i < 8; i++) {
 		log[log_ptr].d[i] = m68k_dreg(regs, i);
 		log[log_ptr].a[i] = m68k_areg(regs, i);
@@ -118,6 +145,11 @@ void m68k_record_step(uaecptr pc)
 	log[log_ptr].sr = regs.sr;
 	log[log_ptr].msp = regs.msp;
 	log[log_ptr].isp = regs.isp;
+	TRY(prb) {
+		log[log_ptr].instr = GET_OPCODE;
+	} CATCH(prb) {
+		log[log_ptr].instr = M68K_EMUL_OP_MAX;
+	}
 	log_ptr = (log_ptr + 1) % LOG_SIZE;
 }
 
@@ -133,6 +165,7 @@ static void dump_log(void)
 		fprintf(f, "d4 %08x d5 %08x d6 %08x d7 %08x\n", log[j].d[4], log[j].d[5], log[j].d[6], log[j].d[7]);
 		fprintf(f, "a0 %08x a1 %08x a2 %08x a3 %08x\n", log[j].a[0], log[j].a[1], log[j].a[2], log[j].a[3]);
 		fprintf(f, "a4 %08x a5 %08x a6 %08x a7 %08x\n", log[j].a[4], log[j].a[5], log[j].a[6], log[j].a[7]);
+		fprintf(f, "instr: %04x\n", log[j].instr);
 #if ENABLE_MON
 		disass_68k(f, log[j].pc);
 #endif
@@ -141,11 +174,19 @@ static void dump_log(void)
 }
 #endif /* FLIGHT_RECORDER */
 
+#if ENABLE_MON
+static void dump_regs(void)
+{
+	m68k_dumpstate(NULL);
+}
+#endif
+
+
 int broken_in;
 
 static inline unsigned int cft_map (unsigned int f)
 {
-#ifndef HAVE_GET_WORD_UNSWAPPED
+#if ((!defined(HAVE_GET_WORD_UNSWAPPED)) || (defined(FULLMMU)))
     return f;
 #else
     return do_byteswap_16(f);
@@ -223,6 +264,9 @@ void init_m68k (void)
 void exit_m68k (void)
 {
 	fpu_exit ();
+
+	free(table68k);
+	table68k = NULL;
 }
 
 struct regstruct regs, lastint_regs;
@@ -236,17 +280,24 @@ int lastint_no;
 static inline uae_u8 get_ibyte_1(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o + 1;
-    return phys_get_byte(mmu_translate(addr, FC_INST, 0, addr, sz_byte, 0));
+    return phys_get_byte(mmu_translate(addr, FC_INST, 0, sz_byte, 0));
 }
 static inline uae_u16 get_iword_1(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o;
-    return phys_get_word(mmu_translate(addr, FC_INST, 0, addr, sz_word, 0));
+    return phys_get_word(mmu_translate(addr, FC_INST, 0, sz_word, 0));
 }
 static inline uae_u32 get_ilong_1(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o;
-    return phys_get_long(mmu_translate(addr, FC_INST, 0, addr, sz_long, 0));
+    if (is_unaligned(addr, 4)) {
+	uae_u32 result;
+	result = phys_get_word(mmu_translate(addr, FC_INST, 0, sz_word, 0));
+	result <<= 16;
+	result |= phys_get_word(mmu_translate(addr + 2, FC_INST, 0, sz_word, 0));
+	return result;
+    }
+    return phys_get_long(mmu_translate(addr, FC_INST, 0, sz_long, 0));
 }
 #else
 # define get_ibyte_1(o) get_byte(m68k_getpc() + (o) + 1)
@@ -709,7 +760,9 @@ static inline void exc_make_frame(
 
 extern void showBackTrace(int, bool=true);
 
+#ifdef EXCEPTIONS_VIA_LONGJMP
 static int building_bus_fault_stack_frame=0;
+#endif
 
 void Exception(int nr, uaecptr oldpc)
 {
@@ -728,37 +781,55 @@ void Exception(int nr, uaecptr oldpc)
 #endif
         // panicbug("Exception Nr. %d CPC: %08lx NPC: %08lx SP=%08lx Addr: %08lx", nr, currpc, get_long (regs.vbr + 4*nr), m68k_areg(regs, 7), regs.mmu_fault_addr);
 
-        if (building_bus_fault_stack_frame) {
+#ifdef EXCEPTIONS_VIA_LONGJMP
+	if (!building_bus_fault_stack_frame)
+#else
+	try
+#endif
+	{
+#ifdef EXCEPTIONS_VIA_LONGJMP
+            building_bus_fault_stack_frame= 1;
+#endif
+	    /* 68040 */
+	    exc_push_long(0);	/* PD3 */
+	    exc_push_long(0);	/* PD2 */
+	    exc_push_long(0);	/* PD1 */
+	    exc_push_long(0);	/* PD0/WB1D */
+	    exc_push_long(0);	/* WB1A */
+	    exc_push_long(0);	/* WB2D */
+	    exc_push_long(0);	/* WB2A */
+	    exc_push_long(regs.wb3_data);	/* WB3D */
+	    exc_push_long(regs.mmu_fault_addr);	/* WB3A */
+	    exc_push_long(regs.mmu_fault_addr);
+	    exc_push_word(0);	/* WB1S */
+	    exc_push_word(0);	/* WB2S */
+	    exc_push_word(regs.wb3_status);	/* WB3S */
+	    regs.wb3_status = 0;
+	    exc_push_word(regs.mmu_ssw);
+#ifdef PUREC
+	    exc_push_long(0 /* was regs.mmu_fault_addr */);	/* EA *//* bullshit here, took 10 hours to debug with PureC $12345678 test. It should be an internal register, keep 0 to preserve MSP in PureC */
+#else
+	    exc_push_long(regs.mmu_fault_addr); /* EA *//* ARAnyM has 040, so stack format 7, not 000 stack format */
+#endif
+	    exc_make_frame(7, regs.sr, regs.fault_pc, 2, 0, 0);
+
+	}
+#ifdef EXCEPTIONS_VIA_LONGJMP
+	else
+#else
+	catch (m68k_exception)
+#endif
+	{
             report_double_bus_error();
-            building_bus_fault_stack_frame=0;
-            return;
+#ifdef EXCEPTIONS_VIA_LONGJMP
+            building_bus_fault_stack_frame= 0;
+#endif
+	    return;
         }
 
-	building_bus_fault_stack_frame=1;
-
-	/* 68040 */
-	exc_push_long(0);	/* PD3 */
-	exc_push_long(0);	/* PD2 */
-	exc_push_long(0);	/* PD1 */
-	exc_push_long(0);	/* PD0/WB1D */
-	exc_push_long(0);	/* WB1A */
-	exc_push_long(0);	/* WB2D */
-	exc_push_long(0);	/* WB2A */
-	exc_push_long(0);	/* WB3D */
-	exc_push_long(0);	/* WB3A */
-	exc_push_long(regs.mmu_fault_addr);
-	exc_push_word(0);	/* WB1S */
-	exc_push_word(0);	/* WB2S */
-	exc_push_word(0);	/* WB3S */
-	exc_push_word(regs.mmu_ssw);
-#ifdef PUREC
-	exc_push_long(0 /* was regs.mmu_fault_addr */);	/* EA *//* bullshit here, took 10 hours to debug with PureC $12345678 test. It should be an internal register, keep 0 to preserve MSP in PureC */
-#else
-	exc_push_long(regs.mmu_fault_addr); /* EA *//* ARAnyM has 040, so stack format 7, not 000 stack format */
+#ifdef EXCEPTIONS_VIA_LONGJMP
+	building_bus_fault_stack_frame= 0;
 #endif
-	exc_make_frame(7, regs.sr, currpc, 2, 0, 0);
-
-	building_bus_fault_stack_frame=0;
         /* end of BUS ERROR handler */
     } else if (nr == 3) {
 	exc_make_frame(2, regs.sr, last_addr_for_exception_3, nr,
@@ -1162,14 +1233,20 @@ void m68k_reset (void)
     regs.cacr = 0;
     regs.caar = 0;
 #if FLIGHT_RECORDER
-#if ENABLE_MON
-    if (log_ptr == -1) {
-        // Install "log" command in mon
-        mon_add_command("log", dump_log, "log                      Dump m68k emulation log\n");
-    }
+	log_ptr = 0;
+	memset(log, 0, sizeof(log));
 #endif
-    log_ptr = 0;
-    memset(log, 0, sizeof(log));
+
+#if ENABLE_MON
+	static bool first_time = true;
+	if (first_time) {
+		first_time = false;
+		mon_add_command("regs", dump_regs, "regs                    Dump m68k emulator registers\n");
+#if FLIGHT_RECORDER
+		// Install "log" command in mon
+		mon_add_command("log", dump_log, "log                      Dump m68k emulation log\n");
+#endif
+	}
 #endif
 }
 
@@ -1464,7 +1541,7 @@ void m68k_do_execute (void)
     if (SETJMP(loop_env)) return;
 #endif	    
     for (;;) {
-	pc = m68k_getpc();
+	regs.fault_pc = pc = m68k_getpc();
 #ifdef FULL_HISTORY
 #ifdef NEED_TO_DEBUG_BADLY
 	history[lasthist] = regs;
@@ -1478,20 +1555,24 @@ void m68k_do_execute (void)
 	}
 #endif
 
+#ifndef FULLMMU
 #if ARAM_PAGE_CHECK
 	if (((pc ^ pc_page) > ARAM_PAGE_MASK)) {
 	    check_ram_boundary(pc, 2, false);
 	    pc_page = pc;
-	    pc_offset = (uae_u32)get_real_address(pc, 0, sz_word) - pc;
+	    pc_offset = (uintptr)get_real_address(pc, 0, sz_word) - pc;
 	}
 #else
 	check_ram_boundary(pc, 2, false);
+#endif
 #endif
 	opcode = GET_OPCODE;
 #if FLIGHT_RECORDER
 	                m68k_record_step(m68k_getpc());
 #endif
 	(*cpufunctbl[opcode])(opcode);
+	cpu_check_ticks();
+	regs.fault_pc = m68k_getpc();
 
 #ifndef DISDIP
 	if (SPCFLAGS_TEST(SPCFLAG_ALL_BUT_EXEC_RETURN)) {
@@ -1502,67 +1583,42 @@ void m68k_do_execute (void)
     }
 }
 
-#if defined(USE_JIT) && !defined(X86_ASSEMBLY_disable)
-void m68k_compile_execute (void)
-{
-setjmpagain:
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
-	flush_icache(0);
-        Exception(prb, 0);
-    	goto setjmpagain;
-    }
-    for (;;) {
-	if (quit_program > 0) {
-	    if (quit_program == 1) {
-#if FLIGHT_RECORDER
-		dump_log();
-#endif
-		break;
-	    }
-	    quit_program = 0;
-	    m68k_reset ();
-	}
-	m68k_do_compile_execute();
-    }
-}
-#endif
-
 void m68k_execute (void)
 {
 #ifdef USE_JIT
     m68k_execute_depth++;
 #endif
 #ifdef DEBUGGER
-    volatile bool after_exception = false;
+    VOLATILE bool after_exception = false;
 #endif
 
 setjmpagain:
-    int prb = SETJMP(excep_env);
-    if (prb != 0) {
+    TRY(prb) {
+	for (;;) {
+	    if (quit_program > 0) {
+		if (quit_program == 1) {
+#if FLIGHT_RECORDER
+		    dump_log();
+#endif
+		    break;
+		}
+		quit_program = 0;
+		m68k_reset ();
+	    }
+#ifdef DEBUGGER
+	    if (debugging && !after_exception) debug();
+	    after_exception = false;
+#endif
+	    m68k_do_execute();
+	}
+    }
+    CATCH(prb) {
         Exception(prb, 0);
 #ifdef DEBUGGER
 	after_exception = true;
 #endif
     	goto setjmpagain;
     }
-    for (;;) {
-	if (quit_program > 0) {
-	    if (quit_program == 1) {
-#if FLIGHT_RECORDER
-		dump_log();
-#endif
-		break;
-	    }
-	    quit_program = 0;
-	    m68k_reset ();
-	}
-#ifdef DEBUGGER
-	if (debugging && !after_exception) debug();
-	after_exception = false;
-#endif
-	m68k_do_execute();
-    }
 
 #ifdef USE_JIT
     m68k_execute_depth--;
@@ -1659,58 +1715,52 @@ void m68k_disasm (uaecptr addr, uaecptr *nextpc, int cnt)
 }
 
 #ifdef NEWDEBUG
-void newm68k_disasm(FILE *f, uaecptr addr, uaecptr *nextpc, volatile unsigned int cnt)
+void newm68k_disasm(FILE *f, uaecptr addr, uaecptr *nextpc, VOLATILE unsigned int cnt)
 {
     char *buffer = (char *)malloc(80 * sizeof(char));
-    JMP_BUF excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+    SAVE_EXCEPTION;
     strcpy(buffer,"");
-    volatile uaecptr newpc = 0;
+    VOLATILE uaecptr newpc = 0;
     m68kpc_offset = addr - m68k_getpc ();
     if (cnt == 0) {
-        int prb = SETJMP(excep_env);
-        if (prb != 0) {
-            goto setjmpagainx;
-        }
-        char instrname[20],*ccpt;
-        int opwords;
-        uae_u32 opcode;
-        struct mnemolookup *lookup;
-        struct instr *dp;
-        for (opwords = 0; opwords < 5; opwords++) {
-            get_iword_1 (m68kpc_offset + opwords*2);
-        }
-        opcode = get_iword_1 (m68kpc_offset);
-        m68kpc_offset += 2;
-        if (cpufunctbl[cft_map (opcode)] == op_illg_1) {
-            opcode = 0x4AFC;
-        }
-        dp = table68k + opcode;
-        for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
-            ;
-        strcpy (instrname, lookup->name);
-        ccpt = strstr (instrname, "cc");
-        if (ccpt != 0) {
-            strncpy (ccpt, ccnames[dp->cc], 2);
-        }
-        if (dp->suse) {
-            newpc = m68k_getpc () + m68kpc_offset;
-            newpc += ShowEA (dp->sreg, (amodes)dp->smode, (wordsizes)dp->size, buffer);
-            strcpy(buffer,"");
-        }
-        if (dp->duse) {
-            newpc = m68k_getpc () + m68kpc_offset;
-            newpc += ShowEA (dp->dreg, (amodes)dp->dmode, (wordsizes)dp->size, buffer);
-            strcpy(buffer,"");
-        }
+        TRY(prb) {
+	    char instrname[20],*ccpt;
+	    int opwords;
+	    uae_u32 opcode;
+	    struct mnemolookup *lookup;
+	    struct instr *dp;
+	    for (opwords = 0; opwords < 5; opwords++) {
+		get_iword_1 (m68kpc_offset + opwords*2);
+	    }
+	    opcode = get_iword_1 (m68kpc_offset);
+	    m68kpc_offset += 2;
+	    if (cpufunctbl[cft_map (opcode)] == op_illg_1) {
+		opcode = 0x4AFC;
+	    }
+	    dp = table68k + opcode;
+	    for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
+		;
+	    strcpy (instrname, lookup->name);
+	    ccpt = strstr (instrname, "cc");
+	    if (ccpt != 0) {
+		strncpy (ccpt, ccnames[dp->cc], 2);
+	    }
+	    if (dp->suse) {
+		newpc = m68k_getpc () + m68kpc_offset;
+		newpc += ShowEA (dp->sreg, (amodes)dp->smode, (wordsizes)dp->size, buffer);
+		strcpy(buffer,"");
+	    }
+	    if (dp->duse) {
+		newpc = m68k_getpc () + m68kpc_offset;
+		newpc += ShowEA (dp->dreg, (amodes)dp->dmode, (wordsizes)dp->size, buffer);
+		strcpy(buffer,"");
+	    }
+	}
+	CATCH(prb) {}
     } else {
 setjmpagain:
-        int prb = SETJMP(excep_env);
-        if (prb != 0) {
-		fprintf (f, " unknown address\n");
-                goto setjmpagain;
-        }
-	while (cnt-- > 0) {
+	TRY(prb) {
+	    while (cnt-- > 0) {
 		char instrname[20],*ccpt;
 		int opwords;
 		uae_u32 opcode;
@@ -1764,12 +1814,16 @@ setjmpagain:
 		    fprintf (f, " == %08lx", (unsigned long)newpc);
 		fprintf (f, "\n");
 	    }
+	}
+	CATCH(prb) {
+		fprintf (f, " unknown address\n");
+                goto setjmpagain;
+	}
     }
-setjmpagainx:
     if (nextpc)
 	*nextpc = m68k_getpc () + m68kpc_offset;
     free(buffer);
-    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+    RESTORE_EXCEPTION;
 }
 
 #ifdef FULL_HISTORY
@@ -1786,72 +1840,68 @@ void showDisasm(uaecptr addr) {
 		buff[i] = (char *)malloc(80 * sizeof(char));
 		strcpy(buff[i],"");
 	}
-	JMP_BUF excep_env_old;
-	memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
-	volatile uaecptr newpc = 0;
+	SAVE_EXCEPTION;
+	VOLATILE uaecptr newpc = 0;
 	m68kpc_offset = addr - m68k_getpc ();
-	int prb = SETJMP(excep_env);
-	if (prb != 0) {
-		bug("%s%s%s%s%s%s%s%s%s%s%s%s unknown address", sbuffer[0], buff[0],  buff[1],  buff[2], buff[3], buff[4], sbuffer[1], sbuffer[2], sbuffer[3], sbuffer[4], sbuffer[5], sbuffer[6]);
-		free(buffer);
-		for (int i = 0; i < 7; i++) free(sbuffer[i]);
-		for (int i = 0; i < 5; i++) free(buff[i]);
-		return;
-	}
-	char instrname[20],*ccpt;
-	int opwords;
-	uae_u32 opcode;
-	struct mnemolookup *lookup;
-	struct instr *dp;
-	sprintf(sbuffer[0], "%08lx: ", m68k_getpc () + m68kpc_offset);
-	for (opwords = 0; opwords < 5; opwords++) {
-		sprintf (buff[opwords], "%04x ", get_iword_1 (m68kpc_offset + opwords*2));
-	}
-	opcode = get_iword_1 (m68kpc_offset);
-	m68kpc_offset += 2;
-	if (cpufunctbl[cft_map (opcode)] == op_illg_1) {
-		opcode = 0x4AFC;
-	}
-	dp = table68k + opcode;
-	for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
-		    ;
-	strcpy (instrname, lookup->name);
-	ccpt = strstr (instrname, "cc");
-	if (ccpt != 0) {
-		strncpy (ccpt, ccnames[dp->cc], 2);
-	}
-	sprintf (sbuffer[1], "%s", instrname);
-	switch (dp->size){
-		 case sz_byte: sprintf (sbuffer[2], ".B "); break;
-		 case sz_word: sprintf (sbuffer[2], ".W "); break;
-		 case sz_long: sprintf (sbuffer[2], ".L "); break;
-		 default: sprintf (sbuffer[2], "   "); break;
-	}
+	TRY(prb) {
+	    char instrname[20],*ccpt;
+	    int opwords;
+	    uae_u32 opcode;
+	    struct mnemolookup *lookup;
+	    struct instr *dp;
+	    sprintf(sbuffer[0], "%08lx: ", m68k_getpc () + m68kpc_offset);
+	    for (opwords = 0; opwords < 5; opwords++) {
+		    sprintf (buff[opwords], "%04x ", get_iword_1 (m68kpc_offset + opwords*2));
+	    }
+	    opcode = get_iword_1 (m68kpc_offset);
+	    m68kpc_offset += 2;
+	    if (cpufunctbl[cft_map (opcode)] == op_illg_1) {
+		    opcode = 0x4AFC;
+	    }
+	    dp = table68k + opcode;
+	    for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
+			;
+	    strcpy (instrname, lookup->name);
+	    ccpt = strstr (instrname, "cc");
+	    if (ccpt != 0) {
+		    strncpy (ccpt, ccnames[dp->cc], 2);
+	    }
+	    sprintf (sbuffer[1], "%s", instrname);
+	    switch (dp->size){
+		     case sz_byte: sprintf (sbuffer[2], ".B "); break;
+		     case sz_word: sprintf (sbuffer[2], ".W "); break;
+		     case sz_long: sprintf (sbuffer[2], ".L "); break;
+		     default: sprintf (sbuffer[2], "   "); break;
+	    }
 
-	if (dp->suse) {
-		newpc = m68k_getpc () + m68kpc_offset;
-		newpc += ShowEA (dp->sreg, (amodes)dp->smode, (wordsizes)dp->size, buffer);
-		sprintf(sbuffer[3], "%s", buffer);
-		strcpy(buffer,"");
+	    if (dp->suse) {
+		    newpc = m68k_getpc () + m68kpc_offset;
+		    newpc += ShowEA (dp->sreg, (amodes)dp->smode, (wordsizes)dp->size, buffer);
+		    sprintf(sbuffer[3], "%s", buffer);
+		    strcpy(buffer,"");
+	    }
+	    if (dp->suse && dp->duse) sprintf (sbuffer[4], ",");
+	    if (dp->duse) {
+		    newpc = m68k_getpc () + m68kpc_offset;
+		    newpc += ShowEA (dp->dreg, (amodes)dp->dmode, (wordsizes)dp->size, buffer);
+		    sprintf(sbuffer[5], "%s", buffer);
+		    strcpy(buffer,"");
+	    }
+	    if (ccpt != 0) {
+		    if (cctrue(dp->cc)) sprintf (sbuffer[6], " == %08lx (TRUE)", (unsigned long)newpc);
+			    else sprintf (sbuffer[6], " == %08lx (FALSE)", (unsigned long)newpc);
+	    } else if ((opcode & 0xff00) == 0x6100) /* BSR */
+		    sprintf (sbuffer[6], " == %08lx", (unsigned long)newpc);
+
+	    bug("%s%s%s%s%s%s%s%s%s%s%s%s", sbuffer[0], buff[0], buff[1], buff[2], buff[3],  buff[4], sbuffer[1], sbuffer[2], sbuffer[3], sbuffer[4], sbuffer[5], sbuffer[6]);
 	}
-	if (dp->suse && dp->duse) sprintf (sbuffer[4], ",");
-	if (dp->duse) {
-		newpc = m68k_getpc () + m68kpc_offset;
-		newpc += ShowEA (dp->dreg, (amodes)dp->dmode, (wordsizes)dp->size, buffer);
-		sprintf(sbuffer[5], "%s", buffer);
-		strcpy(buffer,"");
+	CATCH(prb) {
+		bug("%s%s%s%s%s%s%s%s%s%s%s%s unknown address", sbuffer[0], buff[0],  buff[1],  buff[2], buff[3], buff[4], sbuffer[1], sbuffer[2], sbuffer[3], sbuffer[4], sbuffer[5], sbuffer[6]);
 	}
-	if (ccpt != 0) {
-		if (cctrue(dp->cc)) sprintf (sbuffer[6], " == %08lx (TRUE)", (unsigned long)newpc);
-			else sprintf (sbuffer[6], " == %08lx (FALSE)", (unsigned long)newpc);
-	} else if ((opcode & 0xff00) == 0x6100) /* BSR */
-		sprintf (sbuffer[6], " == %08lx", (unsigned long)newpc);
-	
-	bug("%s%s%s%s%s%s%s%s%s%s%s%s", sbuffer[0], buff[0], buff[1], buff[2], buff[3],  buff[4], sbuffer[1], sbuffer[2], sbuffer[3], sbuffer[4], sbuffer[5], sbuffer[6]);
 	free(buffer);
 	for (int i = 0; i < 7; i++) free(sbuffer[i]);
 	for (int i = 0; i < 5; i++) free(buff[i]);
-	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
+	RESTORE_EXCEPTION;
 }
 #endif
 #endif
@@ -1875,7 +1925,7 @@ void m68k_dumpstate (uaecptr *nextpc)
 	    (unsigned long)regs.msp, (unsigned long)regs.vbr);
     printf ("T=%d%d S=%d M=%d X=%d N=%d Z=%d V=%d C=%d IMASK=%d TCE=%d TCP=%d\n",
 	    regs.t1, regs.t0, regs.s, regs.m,
-	    GET_XFLG, GET_NFLG, GET_ZFLG, GET_VFLG, GET_CFLG, regs.intmask,
+	    (int)GET_XFLG, (int)GET_NFLG, (int)GET_ZFLG, (int)GET_VFLG, (int)GET_CFLG, regs.intmask,
 	    regs.mmu_enabled, regs.mmu_pagesize);
     printf ("CACR=%08lx CAAR=%08lx  URP=%08lx  SRP=%08lx\n",
             (unsigned long)regs.cacr,
