@@ -39,11 +39,7 @@
 #include "m68k.h"
 #include "memory.h"
 
-#ifdef HAVE_NEW_HEADERS
 # include <csetjmp>
-#else
-# include <setjmp.h>
-#endif
 
 extern int areg_byteinc[];
 extern int imm8_table[];
@@ -96,17 +92,19 @@ extern void REGPARAM2 op_illg (uae_u32) REGPARAM;
 static inline uae_u8 get_ibyte(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o + 1;
-    return phys_get_byte(mmu_translate(addr, FC_INST, 0, addr, sz_byte, 0));
+    return phys_get_byte(mmu_translate(addr, FC_INST, 0, sz_byte, 0));
 }
 static inline uae_u16 get_iword(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o;
-    return phys_get_word(mmu_translate(addr, FC_INST, 0, addr, sz_word, 0));
+    return phys_get_word(mmu_translate(addr, FC_INST, 0, sz_word, 0));
 }
 static inline uae_u32 get_ilong(uae_u32 o)
 {
     uaecptr addr = m68k_getpc() + o;
-    return phys_get_long(mmu_translate(addr, FC_INST, 0, addr, sz_long, 0));
+    if (is_unaligned (addr, 4))
+	return mmu_get_unaligned(addr, FC_INST, 4);
+    return phys_get_long(mmu_translate(addr, FC_INST, 0, sz_long, 0));
 }
 
 #else
@@ -140,7 +138,11 @@ static inline uae_u32 get_ilong_prefetch (uae_s32 o)
 }
 #endif
 
+#ifdef FULLMMU
+#define m68k_incpc(o) (regs.pc += (o))
+#else
 #define m68k_incpc(o) (regs.pc_p += (o))
+#endif
 
 static inline void fill_prefetch_0 (void)
 {
@@ -193,8 +195,10 @@ static inline uae_u32 next_ilong (void)
 
 static inline void m68k_setpc (uaecptr newpc)
 {
+#ifndef FULLMMU
     regs.pc_p = regs.pc_oldp = get_real_address(newpc, 0, sz_word);
-    regs.pc = newpc;
+#endif
+    regs.fault_pc = regs.pc = newpc;
 }
 
 #define m68k_setpc_fast m68k_setpc
@@ -209,15 +213,15 @@ static inline void m68k_do_rts(void)
  
 static inline void m68k_do_bsr(uaecptr oldpc, uae_s32 offset)
 {
+    put_long(m68k_areg(regs, 7) - 4, oldpc);
     m68k_areg(regs, 7) -= 4;
-    put_long(m68k_areg(regs, 7), oldpc);
     m68k_incpc(offset);
 }
  
 static inline void m68k_do_jsr(uaecptr oldpc, uaecptr dest)
 {
+    put_long(m68k_areg(regs, 7) - 4, oldpc);
     m68k_areg(regs, 7) -= 4;
-    put_long(m68k_areg(regs, 7), oldpc);
     m68k_setpc(dest);
 }
 
@@ -230,7 +234,9 @@ static inline void m68k_setstopped (int stop)
 	SPCFLAGS_SET( SPCFLAG_STOP );
 }
 
-#ifdef ARAM_PAGE_CHECK
+#ifdef FULLMMU
+# define GET_OPCODE (get_iword (0))
+#elif defined ARAM_PAGE_CHECK
 # ifdef HAVE_GET_WORD_UNSWAPPED
 #  define GET_OPCODE (do_get_mem_word_unswapped((uae_u16*)(pc + pc_offset)));
 # else
@@ -304,13 +310,20 @@ extern void *op_smalltbl_0_lab[];
 extern void m68k_do_execute(void);
 extern void m68k_execute(void);
 #ifdef USE_JIT
-#ifdef X86_ASSEMBLY_disable
-/* This is generated code */
-extern void (*m68k_compile_execute)(void);
-#else
-extern void m68k_do_compile_execute(void);
 extern void m68k_compile_execute(void);
 #endif
+#ifdef USE_CPU_EMUL_SERVICES
+extern int32 emulated_ticks;
+extern void cpu_do_check_ticks(void);
+
+static inline void cpu_check_ticks(void)
+{
+	if (--emulated_ticks <= 0)
+		cpu_do_check_ticks();
+}
+#else
+#define cpu_check_ticks()
+#define cpu_do_check_ticks()
 #endif
 
 #endif /* NEWCPU_H */
