@@ -1,7 +1,7 @@
 /*
  * videl.cpp - Falcon VIDEL emulation
  *
- * Copyright (c) 2001-2004 ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2005 ARAnyM developer team (see AUTHORS)
  *
  * Authors:
  *  joy		Petr Stehlik
@@ -33,6 +33,7 @@
 #include "hardware.h"
 #include "cpu_emulation.h"
 #include "memory.h"
+#include "host.h"
 #include "hostscreen.h"
 #include "videl.h"
 #include "hardware.h"
@@ -41,9 +42,6 @@
 #define DEBUG 0
 #include "debug.h"
 
-// from host.cpp
-extern HostScreen hostScreen;
-
 #define HW	getHWoffset()
 #define VIDEL_COLOR_REGS_BEGIN	0xff9800
 #define VIDEL_COLOR_REGS_END	0xffa200
@@ -76,8 +74,8 @@ void VIDEL::reset()
 	hostColorsSync = false;
 
 	/* Autozoom */
-	zoomwidth=0;
-	zoomheight=0;
+	zoomwidth=prev_scrwidth=0;
+	zoomheight=prev_scrheight=0;
 	zoomxtable=NULL;
 	zoomytable=NULL;
 
@@ -231,6 +229,19 @@ void VIDEL::updateColors()
 	hostColorsSync = true;
 }
 
+void VIDEL::renderScreen(void)
+{
+	if ( doRender )
+		renderScreenNoFlag();
+
+#ifdef ENABLE_OPENGL
+	if (bx_options.opengl.enabled) {
+		hostScreen.OpenGLUpdate();
+
+		SDL_GL_SwapBuffers();
+	}
+#endif	/* ENABLE_OPENGL */
+}
 
 void VIDEL::renderScreenNoFlag()
 {
@@ -266,7 +277,7 @@ void VIDEL::renderScreenNoFlag()
 	if (!hostScreen.renderBegin())
 		return;
 
-	if (bx_options.video.autozoom) {
+	if (bx_options.autozoom.enabled) {
 		renderScreenZoom();
 	} else {
 		renderScreenNoZoom();
@@ -561,6 +572,7 @@ void VIDEL::renderScreenZoom()
 	int vw	 = getScreenWidth();
 	int vh	 = getScreenHeight();
 	uint16 *fvram = (uint16 *) Atari2HostAddr(this->getVideoramAddress());
+	if ((vw<32) || (vh<32)) return;
 
 	int lineoffset = handleReadW(HW + 0x0e) & 0x01ff; // 9 bits
 	int linewidth = handleReadW(HW + 0x10) & 0x03ff; // 10 bits
@@ -575,7 +587,7 @@ void VIDEL::renderScreenZoom()
 	uint8 *hvram = (uint8 *) hostScreen.getVideoramAddress();
 
 	/* Integer zoom coef ? */
-	if ((bx_options.video.autozoomint) && (scrwidth>vw) && (scrheight>vh)) {
+	if ((bx_options.autozoom.integercoefs) && (scrwidth>=vw) && (scrheight>=vh)) {
 		int coefx = scrwidth/vw;
 		int coefy = scrheight/vh;
 
@@ -588,7 +600,7 @@ void VIDEL::renderScreenZoom()
 	}
 
 	/* New zoom ? */
-	if (zoomwidth != vw) {
+	if ((zoomwidth != vw) || (scrwidth != prev_scrwidth)) {
 		if (zoomxtable) {
 			delete zoomxtable;
 		}
@@ -597,8 +609,9 @@ void VIDEL::renderScreenZoom()
 			zoomxtable[i] = (vw*i)/scrwidth;
 		}
 		zoomwidth = vw;
+		prev_scrwidth = scrwidth;
 	}
-	if (zoomheight != vh) {
+	if ((zoomheight != vh) || (scrheight != prev_scrheight)) {
 		if (zoomytable) {
 			delete zoomytable;
 		}
@@ -607,6 +620,7 @@ void VIDEL::renderScreenZoom()
 			zoomytable[i] = (vh*i)/scrheight;
 		}
 		zoomheight = vh;
+		prev_scrheight = scrheight;
 	}
 
 	cursrcline = -1;
@@ -939,3 +953,7 @@ void VIDEL::renderScreenZoom()
 		}
 	}
 }
+
+/*
+vim:ts=4:sw=4:
+*/
